unit ZeldaTiles;
// PAR SDL Zelda was created using the PAR SDL Framework
// Created by Paul Robello by examples and demos from but not limited to:
// nehe.gamedev.net, www.delphi-jedi.org, romka.demonews.com, www.nutty.org.
// Most code has been heavily modified for speed and integration.
// PAR SDL Framework is an always changing project that may not be backward
// compatable with other projects.
// Questions, comments, ravs, flames email paulr@cal.net

interface

uses SysUtils, sdl, Logger, OpenGL12, ParSdlMain, parsdlgl;

const ViewFov=45;                                                               // this is half of total fov
      ViewDistMAX=15;                                                           // radius of view arc
      MapMaxX = 256;
      MapMaxY = 256;

type TTileInfo=record
   x,y,z : longint;
end;

type TtileArc = array [0..ViewFov*2*ViewDistMax] of TTileInfo;


type tmap=record
   data : array[0..MapMaxX,0..MapMaxY] of byte;
   xmax,ymax : longint;                                                         // map boundryies
   xoff,yoff,zoom : longint;                                                     // map mode offset and zoom factor
   shroud,transition : longint;                                                 // effects for map screen
end;

var tiles_2D, tiles_3D : gluint;	    					// Tile Display List
    tilelook : array[0..360] of TTileArc;                                       // Tile Coord lookup
    ViewBobLookSIN,cameraLookSIN,cameraLookCOS : array[0..360] of glFloat;
    ViewDist : glInt;                                                           // current rendering depth
    skytx,skyty : glfloat;                                                      // sky texture offset for clouds
    map : tmap;
    sunlist : GLUINT;                                                           // sun and compass glCompile list
    ShowMap : boolean = false;

procedure BuildTiles;
procedure BuildTileLookup;
procedure LoadTextures;
procedure drawsky;
procedure drawsun;
procedure loadmap(mapfile : string);

implementation

procedure loadmap(mapfile : string);
var f : text;
    st : string;
    x,y : longint;
begin
   if not fileexists(mapfile) then begin
      Log.LogError(Format('map file : %s not found',[mapfile]), '[LoadMap]');
      PAR_System.TerminateApplication;
      exit;
   end;

   assignfile(f,mapfile);
   {$i-}reset(f);{$i+}
   if ioresult<>0 then begin
      Log.LogError(Format('Error reseting map file : %s',[mapfile]), '[LoadMap]');
      PAR_System.TerminateApplication;
      exit;
   end;

   fillchar(map,sizeof(map),0);
   with map do begin
      yMax:=0;
      while not eof(f) do begin
         readln(f,st);
         xMax:=0;
         while pos(',',st)>0 do begin
            st:=trim(st);
            data[xmax,ymax]:=strtoint(copy(st,1, pos(',',st)-1));
            st:=copy(st,pos(',',st)+1, length(st));
            inc(xmax);
         end;
         inc(ymax);
      end;
      closefile(f);

      for y:=0 to (ymax div 2) -1 do begin
         for x:=0 to xmax do begin
            data[x,y]:=     data[x,ymax-y] xor data[x,y];
            data[x,ymax-y]:=data[x,ymax-y] xor data[x,y];
            data[x,y]:=     data[x,ymax-y] xor data[x,y];
         end;
      end;
   end;
end;

procedure buildGround(size : double;tile:byte);
begin
   glDisable(GL_Lighting);
   case tile of
      5  : materials.SetMaterial('groundbrown');
      36 : materials.SetMaterial('groundwhite');
      48 : materials.SetMaterial('sand');
   end;

   gltranslatef(0,0,-0.5);
   glBegin(GL_QUADS);
      glTexCoord2f(0, 0);  glVertex3f(-size, -size, 0);                         // Texture / Vertex Coord (Bottom Left)}
      glTexCoord2f(1, 0);  glVertex3f( size, -size, 0);                         // Texutre / Vertex Coord (Bottom Right)
      glTexCoord2f(1, 1);  glVertex3f( size,  size, 0);                         // Texture / Vertex Coord (Top Right)
      glTexCoord2f(0, 1);  glVertex3f(-size,  size, 0);                         // Texture / Vertex Coord (Top Left)
   glend;
end;


procedure buildWhiteGroundShadow(size : double);
begin
   if PAR_System.Screen.CanMultiTexture then begin
      glDisable(GL_Lighting);
      gltranslatef(0,0,-0.5);

      glActiveTextureARB(GL_TEXTURE0_ARB);
      glEnable(GL_TEXTURE_2D);
      materials.SetMaterial('groundwhite');

      glActiveTextureARB(GL_TEXTURE1_ARB);
      glEnable(GL_TEXTURE_2D);
      materials.SetMaterial('groundshadow1');

      glBegin(GL_QUADS);
         glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0, 0);glMultiTexCoord2fARB(GL_TEXTURE1_ARB,0, 0);  glVertex3f(-size, -size, 0);     // Texture / Vertex Coord (Bottom Left)}
         glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1, 0);glMultiTexCoord2fARB(GL_TEXTURE1_ARB,1, 0);  glVertex3f( size, -size, 0);     // Texutre / Vertex Coord (Bottom Right)
         glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1, 1);glMultiTexCoord2fARB(GL_TEXTURE1_ARB,1, 1);  glVertex3f( size,  size, 0);     // Texture / Vertex Coord (Top Right)
         glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0, 1);glMultiTexCoord2fARB(GL_TEXTURE1_ARB,0, 1);  glVertex3f(-size,  size, 0);     // Texture / Vertex Coord (Top Left)
      glend;
      glActiveTextureARB(GL_TEXTURE1_ARB);
      glDisable(GL_TEXTURE_2D);
      glActiveTextureARB(GL_TEXTURE0_ARB);
   end else begin
      buildGround(size,36);
   end;
end;

procedure buildBrownGroundShadow(size : double);
begin
   if PAR_System.Screen.CanMultiTexture then begin
      glDisable(GL_Lighting);
      gltranslatef(0,0,-0.5);

      glActiveTextureARB(GL_TEXTURE0_ARB);
      glEnable(GL_TEXTURE_2D);
      materials.SetMaterial('groundbrown');

      glActiveTextureARB(GL_TEXTURE1_ARB);
      glEnable(GL_TEXTURE_2D);
      materials.SetMaterial('groundshadow1');

      glBegin(GL_QUADS);
         glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0, 0);glMultiTexCoord2fARB(GL_TEXTURE1_ARB,0, 0);  glVertex3f(-size, -size, 0);     // Texture / Vertex Coord (Bottom Left)}
         glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1, 0);glMultiTexCoord2fARB(GL_TEXTURE1_ARB,1, 0);  glVertex3f( size, -size, 0);     // Texutre / Vertex Coord (Bottom Right)
         glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1, 1);glMultiTexCoord2fARB(GL_TEXTURE1_ARB,1, 1);  glVertex3f( size,  size, 0);     // Texture / Vertex Coord (Top Right)
         glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0, 1);glMultiTexCoord2fARB(GL_TEXTURE1_ARB,0, 1);  glVertex3f(-size,  size, 0);     // Texture / Vertex Coord (Top Left)
      glend;
      glActiveTextureARB(GL_TEXTURE1_ARB);
      glDisable(GL_TEXTURE_2D);
      glActiveTextureARB(GL_TEXTURE0_ARB);
   end else begin
      buildGround(size,5);
   end;
end;

procedure buildTree(size : glfloat; tile : byte);
begin
   case tile of
      49,50: BuildBrownGroundShadow(size);
      95: BuildWhiteGroundShadow(size);
   end;

   glEnable(GL_Lighting);

   materials.SetMaterial('treebarkwhite');
   case tile of
      49: begin
             glcolor3f(1,0.5,0.1);
             gluCylinder(PARQuadric,0.1,0.01,0.35,6,1);
             materials.SetMaterial('treewhite');
             glcolor3f(0,1,0);
          end;
      50: begin
             glcolor3f(1,0.5,0.1);
             gluCylinder(PARQuadric,0.1,0.01,0.35,6,1);
             materials.SetMaterial('treewhite');
             glcolor3f(1,0.5,0.1);
          end;
      95: begin
             gluCylinder(PARQuadric,0.1,0.01,0.35,6,1);
             materials.SetMaterial('treewhite');
          end;
   end;

   gltranslatef(0,0,0.5);
   if isGeforce then gluSphere(PARQuadric, 0.35, 8, 16) else gluSphere(PARQuadric, 0.35, 6, 6);
end;

procedure buildBoulder(size : glfloat; tile : byte);
var myplane : array[0..3] of glDOUBLE;
begin
   BuildGround(size,5);
   glEnable(GL_Lighting);

   case tile of
      8:  materials.SetMaterial('brownrock1');
      71: materials.SetMaterial('greenrock1');
   end;

   myplane[0]:=0;
   myplane[1]:=0;
   myplane[2]:=1;
   myplane[3]:=0;
   glClipPlane(GL_CLIP_PLANE0,@myplane);
   glEnable(GL_CLIP_PLANE0);


   gltranslatef(0.07,0,-0.2);
   gluSphere(PARQuadric, 0.58, 5, 4);

   gltranslatef(0.07,-0.1,0.1);
   gluSphere(PARQuadric, 0.58, 5, 4);

   glDisable(GL_CLIP_PLANE0);
end;

procedure buildbridgewoodlr(size : double;tile : byte);
begin
   glDisable(GL_Lighting);
   case tile of
      68 : begin
              materials.SetMaterial('bridgewoodlr');
           end;
      87 : begin
              materials.SetMaterial('bridgewoodlr');
              glrotatef(-90,0,0,1);
           end;
   end;
   gltranslatef(0,0,-0.5);
   glBegin(GL_QUADS);
      glTexCoord2f(0, 0);  glVertex3f(-size, -size, 0);                         // Texture / Vertex Coord (Bottom Left)}
      glTexCoord2f(1, 0);  glVertex3f( size, -size, 0);                         // Texutre / Vertex Coord (Bottom Right)
      glTexCoord2f(1, 1);  glVertex3f( size,  size, 0);                         // Texture / Vertex Coord (Top Right)
      glTexCoord2f(0, 1);  glVertex3f(-size,  size, 0);                         // Texture / Vertex Coord (Top Left)
   glend;
end;

procedure buildWater(size : double; tile : byte);
var wt : string;
begin
{   case tile of
      9,14,15 : begin
                   wt:='waterc';
                end;
      52,53,10,77 : begin
                       wt:='watert';
                    end;
      82 : begin
              wt:='watert';
              glrotatef(90,0,0,1);
           end;
      80,75 : begin
                 wt:='watert';
                 glrotatef(180,0,0,1);
              end;
      78,79 : begin
                 wt:='watert';
                 glrotatef(-90,0,0,1);
              end;
      12,86,90 : begin
                 wt:='watertr';
                 glrotatef(-90,0,0,1);
              end;
      83,88,84,85 : begin
                 wt:='watertr';
                 glrotatef(180,0,0,1);
              end;
      13,81 : begin
                 wt:='watertr';
                 glrotatef(90,0,0,1);
              end;
      11,76,89 : begin
                 wt:='watertr';
              end;

      54,55 : begin //Fairy
                 wt:='waterc';
              end;
      101 : begin
               wt:='waterc';
            end;
      else begin
         wt:='waterc';
      end;
   end;
}
   glDisable(GL_Lighting);
   wt:='waterc';
   gltranslatef(0,0,-0.5);

      glEnable(GL_TEXTURE_2D);
      materials.SetMaterial(wt);
      glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);         		// Set The Texture Generation Mode For S To Sphere Mapping (NEW)
      glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP); 	        	// Set The Texture Generation Mode For T To Sphere Mapping (NEW)
      glEnable(GL_TEXTURE_GEN_S);
      glEnable(GL_TEXTURE_GEN_T);

      glBegin(GL_TRIANGLE_FAN);
{         glTexCoord2f(0.5, 0.5);} glVertex3f(0, 0, 0.02);                      // Texture / Vertex Coord (center)
{         glTexCoord2f(0, 0);}     glVertex3f(-size, -size, 0);                 // Texture / Vertex Coord (Bottom Left)
{         glTexCoord2f(1, 0);}     glVertex3f( size, -size, 0);                 // Texutre / Vertex Coord (Bottom Right)
{         glTexCoord2f(1, 1);}     glVertex3f( size,  size, 0);                 // Texture / Vertex Coord (Top Right)
{         glTexCoord2f(0, 1);}     glVertex3f(-size,  size, 0);                 // Texture / Vertex Coord (Top Left)
{         glTexCoord2f(0, 0);}     glVertex3f(-size, -size, 0);                 // Texture / Vertex Coord (Bottom Left)
      glend;

      glDisable(GL_TEXTURE_GEN_S);
      glDisable(GL_TEXTURE_GEN_T);
end;

procedure buildMountain(size:glfloat;tile:byte);
var myplane : array[0..3] of glDOUBLE;
begin
   materials.SetMaterial('treebarkwhite');
   case tile of
      0,1,2,3,4,70:begin
           BuildGround(size,5);
          glcolor4f(0.99,0.50,0.1,1);
      end;
      61..66: begin
         BuildGround(size,5);
         glcolor4f(0,1,0,1);
      end;
      94,69,37..39,35:begin
         BuildGround(size,36);
         glcolor4f(0.9,0.9,0.9,1);
      end;
   end;

   glEnable(GL_Lighting);
   myplane[0]:=0;
   myplane[1]:=0;
   myplane[2]:=1;
   myplane[3]:=0;
   glClipPlane(GL_CLIP_PLANE0,@myplane);

   glscalef(1,1,2);
   glEnable(GL_CLIP_PLANE0);
   if isGeforce then gluSphere(PARQuadric, 0.52, 4, 10) else gluSphere(PARQuadric, 0.52, 4, 5);
   glDisable(GL_CLIP_PLANE0);
end;

procedure BuildTiles;
var cx,cy,cx2,cy2,size : double;
    loop : byte;
begin
   tiles_2D:=glGenLists(112);							// Create 111 Tile Display Lists
   tiles_3D:=glGenLists(112);							// Create 111 Tile Display Lists
   glMatrixMode(GL_MODELVIEW);
   glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
   glEnable(GL_TEXTURE_2D);
   glDisable(GL_BLEND);
   glEnable(GL_DEPTH_TEST);
   glEnable(GL_COLOR_MATERIAL);

   size:=1/2;
   cx2:=1/16;
   cy2:=1/8;
   for loop:=0 to 111 do begin
      cx:= (loop mod 16) / 16.0;
      cy:= (loop  div  16) /  8.0;

      glNewList(tiles_2D+loop,GL_COMPILE);
         glDisable(GL_CULL_FACE);
         materials.SetMaterial('overworld');
         glDisable(GL_Lighting);
         glRotatef(180,0,1,0);
         glBegin(GL_QUADS);
            glTexCoord2f(cx+0.001, 1-cy-cy2 );  glVertex3f(-0.5,-0.5,0);        // Texture / Vertex Coord (Bottom Left)}
            glTexCoord2f(cx+cx2  , 1-cy-cy2 );  glVertex3f( 0.5,-0.5,0);        // Texutre / Vertex Coord (Bottom Right)
            glTexCoord2f(cx+cx2  , 0.9999-cy);  glVertex3f( 0.5, 0.5,0);        // Texture / Vertex Coord (Top Right)
            glTexCoord2f(cx+0.001, 0.9999-cy);  glVertex3f(-0.5, 0.5,0);        // Texture / Vertex Coord (Top Left)
         glEnd();
         glEnable(GL_CULL_FACE);
      glEndList;

      glNewList(tiles_3D+loop,GL_COMPILE);
         glEnable(GL_CULL_FACE);
         materials.SetMaterial('overworld');
         glEnable(GL_Lighting);
         case loop of
             6,7,41,67: begin
                glDisable(GL_Lighting);
                glBegin(GL_QUADS);
                   glTexCoord2f(cx+0.001, 1-cy-cy2 );  glVertex3f(-size, -size, -size);// Texture / Vertex Coord (Bottom Left)}
                   glTexCoord2f(cx+cx2  , 1-cy-cy2 );  glVertex3f( size, -size, -size);// Texutre / Vertex Coord (Bottom Right)
                   glTexCoord2f(cx+cx2  , 0.9999-cy);  glVertex3f( size,  size, -size);// Texture / Vertex Coord (Top Right)
                   glTexCoord2f(cx+0.001, 0.9999-cy);  glVertex3f(-size,  size, -size);// Texture / Vertex Coord (Top Left)
                glEnd();
             end;
             0..4,35,37..39,61..66,69,70,94: begin
                   buildMountain(size,loop);
                end;
             5,36,48: begin
                   buildGround(size,loop);
                end;
             9..15,75..86,88..90,101,52..55: begin
                   buildWater(size, loop);
                end;
            49,50,95: begin
                   buildTree(size,loop);
                end;
            68,87: begin
                   buildbridgewoodlr(size,loop);
                end;
            71,8: begin
                   buildBoulder(size,loop);
                end;
           else begin
              glDisable(GL_Lighting);
              glDisable(GL_CULL_FACE);
              glBegin(GL_QUADS);
                 {top}
                 glNormal3f (0.0, 1.0, 0);
                 glTexCoord2f(cx+0.001, 0.9999-cy); glVertex3f (-size, size, size);
                 glTexCoord2f(cx+cx2  , 0.9999-cy); glVertex3f ( size, size, size);
                 glTexCoord2f(cx+cx2  , 1-cy-cy2 ); glVertex3f ( size,-size, size);
                 glTexCoord2f(cx+0.001, 1-cy-cy2 ); glVertex3f (-size,-size, size);
                 {right}
                 glNormal3f (1, 0.0, 0.0);
                 glTexCoord2f(cx+0.001, 0.9999-cy); glVertex3f (size,-size, size);
                 glTexCoord2f(cx+cx2  , 0.9999-cy); glVertex3f (size, size, size);
                 glTexCoord2f(cx+cx2  , 1-cy-cy2 ); glVertex3f (size, size,-size);
                 glTexCoord2f(cx+0.001, 1-cy-cy2 ); glVertex3f (size,-size,-size);
                 {left}
                 glNormal3f (-1, 0.0, 0.0);
                 glTexCoord2f(cx+0.001, 0.9999-cy); glVertex3f (-size,-size, size);
                 glTexCoord2f(cx+cx2  , 0.9999-cy); glVertex3f (-size, size, size);
                 glTexCoord2f(cx+cx2  , 1-cy-cy2 ); glVertex3f (-size, size,-size);
                 glTexCoord2f(cx+0.001, 1-cy-cy2 ); glVertex3f (-size,-size,-size);
                 {front}
                 glNormal3f (0.0, -1, 0.0);
                 glTexCoord2f(cx+0.001, 0.9999-cy); glVertex3f (-size,-size, size);
                 glTexCoord2f(cx+cx2  , 0.9999-cy); glVertex3f ( size,-size, size);
                 glTexCoord2f(cx+cx2  , 1-cy-cy2 ); glVertex3f ( size,-size,-size);
                 glTexCoord2f(cx+0.001, 1-cy-cy2 ); glVertex3f (-size,-size,-size);
                 {back}
                 glNormal3f (0.0, 1, 0.0);
                 glTexCoord2f(cx+0.001, 0.9999-cy); glVertex3f (-size, size, size);
                 glTexCoord2f(cx+cx2  , 0.9999-cy); glVertex3f ( size, size, size);
                 glTexCoord2f(cx+cx2  , 1-cy-cy2 ); glVertex3f ( size, size,-size);
                 glTexCoord2f(cx+0.001, 1-cy-cy2 ); glVertex3f (-size, size,-size);
               glEnd;
               glEnable(GL_CULL_FACE);
            end;
         end;
         glEndList;
   end;
end;

procedure LoadTextures;
var newMaterial : PMaterial;
begin
   new(newmaterial);
   newMaterial.Name:='sky1';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='sky1.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),0);

   new(newmaterial);
   newMaterial.Name:='heart1';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='heart1.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),0);

   new(newmaterial);
   newMaterial.Name:='heart2';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='heart2.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),0);

   new(newmaterial);
   newMaterial.Name:='heart3';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='heart3.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),0);

   new(newmaterial);
   newMaterial.Name:='sun';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='sun.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),0);

   new(newmaterial);
   newMaterial.Name:='compass1';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='compass1.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),0);

   new(newmaterial);
   newMaterial.Name:='compass2';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='compass2.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),0);

   new(newmaterial);
   newMaterial.Name:='compass3';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='compass2.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),0);

   new(newmaterial);
   newMaterial.Name:='compass4';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='compass4.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),0);

   new(newmaterial);
   newMaterial.Name:='lightmap1';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='lightmap1.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),0);

   new(newmaterial);
   newMaterial.Name:='overworld';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='world.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),1);

   new(newmaterial);
   newMaterial.Name:='groundbrown';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='groundbrown.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),0);

   new(newmaterial);
   newMaterial.Name:='groundwhite';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='groundwhite.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),1);

   new(newmaterial);
   newMaterial.Name:='groundshadow1';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='groundshadow1.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),1);

   new(newmaterial);
   newMaterial.Name:='treebarkwhite';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='treebarkwhite.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),1);

   new(newmaterial);
   newMaterial.Name:='treewhite';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='treewhite.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),1);

   new(newmaterial);
   newMaterial.Name:='greenrock1';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='greenrock1.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),0);

   new(newmaterial);
   newMaterial.Name:='brownrock1';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='brownrock1.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),0);

   new(newmaterial);
   newMaterial.Name:='waterc';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='waterc.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),0);

   new(newmaterial);
   newMaterial.Name:='watert';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='watert.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),0);

   new(newmaterial);
   newMaterial.Name:='watertr';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='watertr.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),0);

   new(newmaterial);
   newMaterial.Name:='bridgewoodlr';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='bridgewoodlr.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),1);

   new(newmaterial);
   newMaterial.Name:='sand';
   newMaterial.Diffuse[0]:=1;
   newMaterial.Diffuse[1]:=1;
   newMaterial.Diffuse[2]:=1;
   newMaterial.Diffuse[3]:=1;
   newMaterial.Specular:=newMaterial.Diffuse;
   newMaterial.TextureFileName:='sand.tga';
   Materials.LoadGLTexture(Materials.AddMaterial(newMaterial),1);
end;

//create a
procedure BuildTileLookup();
var dir,r,d,x,y,tx,ty,z,ox,oy,td,flag : longint;
begin
   fillchar(tilelook,sizeof(tilelook),0);                                       // clear out old lookup tables
   fillchar(CameraLookCOS,sizeof(CameraLookCOS),0);
   fillchar(CameraLookSIN,sizeof(CameraLookSIN),0);
   fillchar(ViewBobLookSIN,sizeof(ViewBobLookSIN),0);

   x:=1000;                                                                     // make sure oldx and oldy update
   y:=1000;
   for dir:=0 to 360 do begin
      CameraLookCOS[dir]:=-PAR_System.COS_LOOK[dir]*16;                         // pre compute camera look
      CameraLookSIN[dir]:=-PAR_System.SIN_LOOK[dir]*16;
      ViewBobLookSIN[dir]:=0.15+abs(PAR_System.SIN_LOOK[dir]*0.05);             // pre compute view bob

      for r:=ViewDist downto 0 do begin
         for d:=dir-ViewFov to dir+ViewFov do begin
            td:=d;
            if td<0 then td:=td+360;;
            if td>360 then td:=td-360;
            ox:=x;
            oy:=y;
            x:=-trunc(PAR_System.COS_LOOK[td]*r);                               // 3d space x
            y:= trunc(PAR_System.SIN_LOOK[td]*r);                               // 3d space y
            if (x=ox) and (y=oy) then continue;                                 // dont repeat same tile
            flag:=0;
            for z:=1 to tilelook[dir][0].x do begin
               if (x=tilelook[dir][z].x) and  (y=tilelook[dir][z].y) then begin
                  flag:=1;
                  break;
               end;
            end;

            if flag=0 then begin                                                // if we dont have it in our list add it
               tilelook[dir][0].x:=tilelook[dir][0].x+1;
               tilelook[dir][tilelook[dir][0].x].x:=x;
               tilelook[dir][tilelook[dir][0].x].y:=y;
            end;
         end;
      end;
      for tx:=-2 to 2 do begin                                                  // fixes missing tile in bottom corners problem
         for ty:=-2 to 2 do begin
            flag:=0;
            for z:=1 to tilelook[dir][0].x do begin
               if (tx=tilelook[dir][z].x) and  (ty=tilelook[dir][z].y) then begin
                  flag:=1;
                  break;
               end;
            end;

            if flag=0 then begin                                                // if we dont have it in our list add it
               tilelook[dir][0].x:=tilelook[dir][0].x+1;
               tilelook[dir][tilelook[dir][0].x].x:=tx;
               tilelook[dir][tilelook[dir][0].x].y:=ty;
            end;
         end;
      end;
   end;
end;

procedure drawsky;
var size : longint;
begin
   size:=5;
   materials.SetMaterial('sky1');
   glDisable(GL_LIGHTING);
   glDisable(GL_DEPTH_TEST);
   glDisable(GL_CULL_FACE);                                                     // You don't need those in 2D.
   glDepthMask(false);
   glblendfunc(GL_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
   glEnable(GL_BLEND);
   glEnable(GL_TEXTURE_2D);

      glLoadIdentity;

      glrotatef(-90,1,0,0);

      skytx:=skytx+0.001;                                                       // scroll sky texture
      skyty:=skyty+0.001;
      if skytx>1 then skytx:=skytx-1;
      if skyty>1 then skyty:=skyty-1;

      glBegin(GL_TRIANGLE_FAN);                                                 // draw sky
         glcolor4f(1,1,1,1); glTexCoord2f(0+skytx/2, 0+skyty/2);  glVertex3f(0, 0, 0.5);             // Texture / Vertex Coord (center)
         glcolor4f(1,1,1,0); glTexCoord2f(0+skytx, 0+skyty);  glVertex3f(-size, -size, 0.5);         // Texture / Vertex Coord (Bottom Left)
         glcolor4f(1,1,1,0); glTexCoord2f(1+skytx, 0+skyty);  glVertex3f( size, -size, 0.5);         // Texutre / Vertex Coord (Bottom Right)
         glcolor4f(1,1,1,0); glTexCoord2f(1+skytx, 1+skyty);  glVertex3f( size,  size, 0.5);         // Texture / Vertex Coord (Top Right)
         glcolor4f(1,1,1,0); glTexCoord2f(0+skytx, 1+skyty);  glVertex3f(-size,  size, 0.5);         // Texture / Vertex Coord (Top Left)
         glcolor4f(1,1,1,0); glTexCoord2f(0+skytx, 0+skyty);  glVertex3f(-size, -size, 0.5);         // Texture / Vertex Coord (Bottom Left)
      glend;

   glDepthMask(true);
   glDisable(GL_BLEND);
   glEnable(GL_CULL_FACE);
end;

procedure drawsun;
begin
   glPushMatrix();
   gltranslatef(light0.pos[0],light0.pos[1],light0.pos[2]);

   if not glIslist(sunlist) then begin
      sunlist:=glGenLists(1);							
      glNewList(sunlist,GL_COMPILE_AND_EXECUTE);
         glDisable(GL_TEXTURE_2D);
         glcolor4f(0.85,0.75,1,0.5);
         gluSphere(PARQuadric, 0.15, 6, 6);
         glEnable(GL_TEXTURE_2D);
      GLEndList;
   end else GLCallList(sunlist);
   glPopMatrix();
end;


end.
