unit flaredef;
// PAR SDL Zelda was created using the PAR SDL Framework
// Created by Paul Robello by examples and demos from but not limited to:
// nehe.gamedev.net, www.delphi-jedi.org, romka.demonews.com, www.nutty.org.
// Most code has been heavily modified for speed and integration.
// PAR SDL Framework is an always changing project that may not be backward
// compatable with other projects.
// Questions, comments, ravs, flames email paulr@cal.net

interface

uses sysutils,opengl12,sdl,logger,ParSdlMain,parsdlgl;

var  doFlare : Boolean = true;

procedure calcflare;

implementation

procedure calcflare;
var modelMatrix,projmatrix : tmatrix4d;
    x,y : longint;                                                              // Used for going through the depth buffer.
    loop : glint;
    big : boolean;
    viewport : tvector4i;                                                       // The viewport.
    depth : glfloat;                                                            // The depth in the framebuffer.
    brightness :  glfloat;                                                      // How bright the light is.
    sx,sy,sz,vpx,vpy : gldouble;                                                // Where on the screen the light is.

     // This is used for making the flares. It only works on one part of the program. Add some more if you want, but keep them with the rest.
     // P is the position. 1.0 is where the light is, -1.0 is the opposite side.
     // S is the size. 1.0 will fill the screen when looking right at the light.
     // R,G, and B are the colour of the flare.

   procedure make_flare(p,s,r,g,b:glFloat);
   var loop : glint;
   begin
      glBegin(GL_TRIANGLE_FAN);
      glColor4f(1.0,1.0,1.0,brightness*0.5);
      glVertex2f(sx*P,1.5*sy*P);
      glColor4f(R,G,B,0);
      for loop:=0 to 6 do begin
         glVertex2f(sx*P+PAR_System.SIN_look[loop*60]*S*brightness,1.5*sy*P+PAR_System.COS_look[loop*60]*S*brightness);
      end;
      glEnd();
   end;

begin
   glLoadIdentity();
   setcamera(@camera);
//   glDisable(GL_DEPTH_TEST);                                                  // disabled per compass
//   glDisable(GL_CULL_FACE);                                                   // disables per compass
//   glDisable(GL_LIGHTING);                                                    // disabled per compass
//   if fog.Enabled then glDisable(GL_FOG);                                     // disabled per compass
//   glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);                          // enabled per compass
//   glEnable(GL_BLEND);                                                        // enabled per compass
   glDisable(GL_TEXTURE_2D);

   brightness:=0;                                                               // How bright the light is.

   glGetDoublev(GL_MODELVIEW_MATRIX, @modelMatrix);                             // Load the matricies and viewport.
   glGetDoublev(GL_PROJECTION_MATRIX, @projMatrix);
   glGetIntegerv(GL_VIEWPORT, @viewport);

   gluProject(light0.pos[0],light0.pos[1],light0.pos[2],modelMatrix,projMatrix,viewport,@sx,@sy,@sz); // Find out where the light is on the screen.

   for x:=-4 to 4 do begin // Go through some of the points near the light.
      vpx:=viewport[2]/300.0*x+sx;
      for y:=-4 to 4 do begin
         vpy:=viewport[3]/300.0*y+sy;
         if((vpx < viewport[2]) and (vpx >= 0) and (vpy < viewport[3]) and (vpy >= 0) ) then begin // If the point is on the screen
            glReadPixels(trunc(vpx),trunc(vpy),1,1,GL_DEPTH_COMPONENT,GL_FLOAT,@depth); // Read the depth from the depth buffer.
            if(depth >= sz) then  brightness := brightness+1.0/81.0;            // If the light is infront of what ever was in that spot, increase the brightness.
         end;
      end;
   end;

   if( (sx < viewport[2]) and (sx >= 0) and (sy < viewport[3]) and (sy >= 0) ) then begin // If the light is on the screen
      glReadPixels(trunc(sx),trunc(sy),1,1,GL_DEPTH_COMPONENT,GL_FLOAT,@depth);
      if(depth < sz) then brightness := 0;                                      // but it is behind something. The light can't be seen.
   end else exit;                                                               // If the light isn't on the screen The light can't be seen.

   sx := sx/viewport[2]*2.0-1.0;                                                // Now that we know the brightness, convert
   sy := sy/viewport[2]*2.0-1.0;                                                // The position to a nember between (-1,-1) and (1,1)

   brightness := (1.2-sqrt(sx*sx+sy*sy))*brightness*0.75;                       // And adjust the brightness so it is brighter at the centre of the screen.

   glLoadIdentity();


   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
      glLoadIdentity();                                                         // Forget about the view and projection. We will be drawing in 2D for the flares.
      gltranslatef(0,0.5,0);

      glBegin(GL_TRIANGLE_FAN);                                                 // Make a steak across the screen where the light is.
         glColor4f(0.9,1.0,0.9,brightness*0.5);
         glVertex2f(sx,sy);
         glColor4f(1.0,1.0,1.0,0);
         glVertex2f(sx+2.5*brightness,sy);
         glVertex2f(sx,sy+0.1*brightness);
         glVertex2f(sx-2.5*brightness,sy);
         glVertex2f(sx,sy-0.1*brightness);
         glVertex2f(sx+2.5*brightness,sy);
      glEnd();

{      glColor4f(1,1,1,brightness/4);                                           // Brighten the screen.
      glBegin(GL_QUADS);
         glTexCoord2f(0,0);
         glVertex2f(sx-2,sy-2);
         glTexCoord2f(1,0);
         glVertex2f(sx+2,sy-2);
         glTexCoord2f(1,1);
         glVertex2f(sx+2,sy+2);
         glTexCoord2f(0,1);
         glVertex2f(sx-2,sy+2);
      glEnd();
}
      big := true;
      glBegin(GL_TRIANGLE_FAN);
         glColor4f(1.0,1.0,1.0,brightness*0.5);
         glVertex2f(sx,sy);

         glColor4f(1.0,1.0,0.5,0);
         for loop:=0 to 60 do begin                                             // Make a 32 point star (64/2=32). Feel free to change that if you want.
            if big then glVertex2f(sx+PAR_System.SIN_look[loop*6]*(brightness),sy+PAR_System.COS_look[loop*6]*(brightness))
               else glVertex2f(sx+PAR_System.SIN_look[loop*6]*(0.5*brightness),sy+PAR_System.COS_look[loop*6]*(0.5*brightness));
            big := not big;                                                     // Make the next part of the star the opposite of this part.
         end;
      glEnd();

     gltranslatef(0,-0.1,0);

      MAKE_FLARE( 0.6,0.3, 1.0, 0.6, 0.6);                                      // Make the flares. Add or Remove some if you like. The MAKE_FLARE macro will only work right here.
      MAKE_FLARE( 0.4,0.4, 1.0, 1.0, 1.0);                                      // It looks better with a chaotic size. I don't know why, though.
      MAKE_FLARE( 0.3,0.6, 0.8, 0.8, 0.6);                                      // Have mine coloured to make a rainbow.
      MAKE_FLARE( 0.2,0.5, 1.0, 1.0, 1.0);
      MAKE_FLARE(-0.1,0.4, 0.6, 1.0, 0.6);
      MAKE_FLARE(-0.2,0.3, 1.0, 1.0, 1.0);
      MAKE_FLARE(-0.4,0.2, 0.8, 0.4, 0.4);
      MAKE_FLARE(-0.6,0.4, 1.0, 1.0, 1.0);
      MAKE_FLARE(-0.7,0.5, 0.4, 0.9, 0.4);
      MAKE_FLARE(-1.0,0.6, 1.0, 1.0, 1.0);
   glPopMatrix();
   glMatrixMode(GL_MODELVIEW);

end;


end.
