unit monsterdef;
// PAR SDL Zelda was created using the PAR SDL Framework
// Created by Paul Robello by examples and demos from but not limited to:
// nehe.gamedev.net, www.delphi-jedi.org, romka.demonews.com, www.nutty.org.
// Most code has been heavily modified for speed and integration.
// PAR SDL Framework is an always changing project that may not be backward
// compatable with other projects.
// Questions, comments, ravs, flames email paulr@cal.net

interface

uses opengl12,parsdlmain, parsdlgl, zeldatiles, linkdef;

const mstDead=0;                                                                // Monster current states
      mstDying=1;
      mstStop=2;
      mstAttack=3;
      mstMove=4;

      mptStop=0;                                                                // Monster Patterns
      mptMove=1;
      mptMoveRandom=2;
      mptMoveChase=3;
      mptMovePatern=4;

      MonsterMax = 10;                                                          // max active monsters
      MonsterListMax = 10;                                                      // max monster meshes

type tmonster=class(tobject)
public
   name : string;
   pos : tvector3d;
   dir,newdir : longint;
   life : longint;
   speed : glfloat;
   brain : longint;
   state : longint;
   list : gluint;
   constructor create(NewName:string;newx,newy,newz:glfloat;newLife,newState,newBrain:longint);
   destructor destroy; override;
   procedure draw(link:plink); virtual;
end;

type Toctorok=class(tmonster)                                                   // little red/blue rock shooting things
public
   procedure draw(link:plink); override;
end;

type Tmadusa=class(tmonster)                                                    // not in actual game, just looks cool
public
   procedure draw(link:plink); override;
   procedure DrawTentacle(const Step : Integer; const Angle, TwistAngle : glFloat);
end;

var monster : array[0..MonsterMax-1] of tmonster;
    MonsterList : array[0..MonsterListMax-1] of gluint;
    NumMonsters : longint=0;                                                    //how many active monsters
    NumMonsterList : longint=0;                                                 //how many monster lists does not have to be active

procedure buildMonsters;
procedure disposeMonsters;                                                      // free all active monsters and all lists

implementation

constructor TMonster.create(NewName:string;newx,newy,newz:glfloat;newlife,newState,newBrain:longint);
begin
   inherited create;
   name:=NewName;
   pos[0]:=newx;
   pos[1]:=newy;
   pos[2]:=newz;
   dir:=0;
   newdir:=dir;
   life:=newlife;
   speed:=0.02;
   brain:=newBrain;
   state:=newstate;
   list:=0;
end;

destructor TMonster.destroy;
begin
   inherited destroy;
end;

procedure TMonster.draw(link:plink);
begin
//
end;

procedure TOctorok.draw(link : Plink);
var o,a : glfloat;
    t: longint;
begin
   if not PAR_System.doPause then begin;
      if brain=mptMoveRandom then begin
         if random(100)=0 then begin
            newdir:=random(360);
         end;
         if (abs(link.lx-pos[0])<2) and (abs(link.ly-pos[1])<2) then state:=mstStop else state:=mstMove;
      end;

      if brain=mptMoveChase then begin
         a:=pos[0]-link.lx;
         o:=pos[1]-link.ly;

         if a=0 then a:=0.0001;
         newdir:=180+trunc(arctan2(o,a)*M_ONE_80_over_PI);


         if (abs(link.lx-pos[0])<2) and (abs(link.ly-pos[1])<2) then state:=mstStop else state:=mstMove;
      end;

      if (brain=mptMoveChase) or (brain=mptMoveRandom) then begin
         if (dir-(newdir-dir)<newdir) then t:=1 else t:=-1;
         dir:=dir+t;
      end;

      if dir < 0 then dir:=dir+360;
      if dir > 360 then dir:=dir-360;

      if state=mstMove then begin
         pos[0]:=pos[0]+PAR_SYSTEM.COS_LOOK[dir]*speed;
         pos[1]:=pos[1]+PAR_SYSTEM.SIN_LOOK[dir]*speed;
      end;
   end;

   if (abs(link.lx-pos[0])>viewDist) or (abs(link.ly-pos[1])>viewDist) then exit;

   glPushMatrix();
      if fog.Enabled then glEnable(GL_FOG);
      glrotatef(-90,1,0,0);
      glTranslatef(pos[0]-link.lx,pos[1]-link.ly,pos[2]-0.5+ViewBobLookSIN[link.viewbob]); // monsters not attached to grid so undo world translation
      glrotatef(dir,0,0,1);
      glCallList(list);
   glPopMatrix();
end;

procedure buildOctorok(r,g,b:glfloat);
var myplane : array[0..3] of glDOUBLE;
begin
   myplane[0]:=0;
   myplane[1]:=0;
   myplane[2]:=1;
   myplane[3]:=0;

   monsterList[numMonsterList]:=glGenLists(1);
   glpushmatrix();
      glLoadIdentity();

         glNewList(Monsterlist[numMonsterList],GL_COMPILE);
         glClipPlane(GL_CLIP_PLANE0,@myplane);
         glEnable(GL_CLIP_PLANE0);

         glDisable(GL_CULL_FACE);
         glEnable(GL_DEPTH_TEST);
         glDisable(GL_TEXTURE_2D);

         glrotatef(90,0,1,0);
         glcolor4f(0,0,0,1);                                                    // mouth inside
         gltranslatef(0,0,0.12);
         gluCylinder(PARQuadric,0.0,0.06,0.20,8,1);
         gltranslatef(0,0,-0.12);

         glcolor4f(r,g,b,1);                                                  // mouth outside
         gluCylinder(PARQuadric,0.02,0.06,0.32,8,1);

         glcolor4f(r,g,b,1);                                                    //body
         gluSphere(PARQuadric, 0.15, 8, 8);

         gltranslatef(-0.1,0.05,0.05);                                          //right eye
         glcolor4f(1,1,1,1);
         gluSphere(PARQuadric, 0.05, 8, 8);

         gltranslatef(0,-0.1,0);                                                //left eye
         glcolor4f(1,1,1,1);
         gluSphere(PARQuadric, 0.05, 8, 8);

         gltranslatef(0,0,0.05);                                                //left eye puple
         glcolor4f(0,0,0,1);
         gluSphere(PARQuadric, 0.01, 4, 4);

         gltranslatef(0,0.1,0);                                                 //right eye puple
         glcolor4f(0,0,0,1);
         gluSphere(PARQuadric, 0.01, 4, 4);

         glEnable(GL_TEXTURE_2D);
         glDisable(GL_CLIP_PLANE0);
      glEndList();
   glPopmatrix();
   inc(NumMonsterList);
end;

procedure TMadusa.DrawTentacle(const Step : Integer; const Angle, TwistAngle : glFloat);
begin
   if Step = 18 then exit;                                                      // We have reached end of tentacle

   glTranslatef(2.25, 0, 0);

   glRotatef(Angle, 0, 1, 0);
   glRotatef(TwistAngle, 1, 0, 0);

   gluSphere(PARQuadric, 1.2, 10, 10);

   glScalef(0.9, 0.9, 0.9);
   DrawTentacle(Step+1, Angle, TwistAngle);
end;

procedure TMadusa.Draw(link:plink);
var Angle, TwistAngle,DemoTime : glFloat;
begin
   demotime:=PAR_System.timerGetTime;

   if dir < 0 then dir:=dir+360;
   if dir > 360 then dir:=dir-360;

   if state=mstMove then begin
      pos[0]:=pos[0]+PAR_SYSTEM.COS_LOOK[dir]*speed;
      pos[1]:=pos[1]+PAR_SYSTEM.SIN_LOOK[dir]*speed;
   end;
   if (abs(link.lx-pos[0])>viewDist) or (abs(link.ly-pos[1])>viewDist) then exit;


   if fog.Enabled then glEnable(GL_FOG);

   glEnable(GL_TEXTURE_GEN_S);
   glEnable(GL_TEXTURE_GEN_T);
   glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);         		// Set The Texture Generation Mode For S To Sphere Mapping (NEW)
   glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP); 	        	// Set The Texture Generation Mode For T To Sphere Mapping (NEW)

   glPushMatrix();
      glrotatef(-90,1,0,0);
      glTranslatef(pos[0]-link.lx,pos[1]-link.ly,pos[2]-0.5+ViewBobLookSIN[link.viewbob]);
      glrotatef(dir,0,0,1);


   glScalef(0.05, 0.05, 0.05);
   gluSphere(PARQuadric, 1.2, 10, 10);

   Angle :=25*sin(DemoTime/600);                                                 //top arm
   TwistAngle :=25*sin(DemoTime/800);
   glPushMatrix();
      DrawTentacle(0, Angle, TwistAngle);
   glPopMatrix();

//   Angle :=25*cos(DemoTime/500);                                               //bottom arm
//   TwistAngle :=25*sin(DemoTime/800);
   glRotatef(90, 0, 1, 0);
{   glPushMatrix();
      DrawTentacle(0, Angle, -TwistAngle);
   glPopMatrix();
}
   Angle :=25*sin(DemoTime/500);                                                 //front arm
   TwistAngle :=25*cos(DemoTime/600);
   glRotatef(90, 0, 1, 0);
   glPushMatrix();
      DrawTentacle(0, Angle, TwistAngle);
   glPopMatrix();

   Angle :=25*sin(DemoTime/600);                                                 //back arm
   TwistAngle :=25*sin(DemoTime/400);
   glRotatef(90, 0, 1, 0);
   glPushMatrix();
      DrawTentacle(0, Angle, -TwistAngle);
   glPopMatrix();

   Angle :=25*sin(DemoTime/600);                                                 //left arm
   TwistAngle :=25*cos(DemoTime/400);
   glRotatef(90, 0, 0, 1);
   glPushMatrix();
      DrawTentacle(0, Angle, TwistAngle);
   glPopMatrix();

   Angle :=25*sin(DemoTime/400);                                                 //right arm
   TwistAngle :=25*cos(DemoTime/800);
   glRotatef(180, 0, 0, 1);
   glPushMatrix();
      DrawTentacle(0, -Angle, -TwistAngle);
   glPopMatrix();


   glDisable(GL_TEXTURE_GEN_S);
   glDisable(GL_TEXTURE_GEN_T);
   glPopMatrix;
end;

procedure buildMonsters;
begin
  buildOctorok(1,0,0);
  buildOctorok(0,0,1);

  monster[NumMonsters]:=TOctorok.create('Octorok Red',link.lx+(random(6)-3)*2,link.ly+(random(6)-3)*2,0, 1, mptMove, mptMoveChase);
  monster[NumMonsters].list:=monsterList[0];
  inc(NumMonsters);

  monster[NumMonsters]:=TOctorok.create('Octorok Blue',link.lx+(random(6)-3)*2,link.ly+(random(6)-3)*2,0, 2, mptMove, mptMoveChase);
  monster[NumMonsters].list:=monsterList[1];
  inc(NumMonsters);

  monster[NumMonsters]:=TMadusa.create('Madusa Blue',133,42,0, 1, mptStop, mptStop);
  inc(NumMonsters);
end;

procedure disposeMonsters;
var loop : longint;
begin
   for loop :=0 to numMonsters-1 do begin
      if assigned(monster[loop]) then monster[loop].destroy;
   end;
   for loop :=0 to numMonsterList-1 do begin
      if glisList(monsterlist[loop]) then glDeleteLists(monsterlist[loop],1);
   end;
end;

initialization

finalization

end.
