unit parsdlgl;
// This file is part of the PAR SDL Framework
// Created by Paul Robello by examples and demos from but not limited to:
// nehe.gamedev.net, www.delphi-jedi.org, romka.demonews.com, www.nutty.org.
// Most code has been heavily modified for speed and integration.
// PAR SDL Framework is an always changing project that may not be backward
// compatable with other projects.
// Questions, comments, ravs, flames email paulr@cal.net

interface

uses  Windows,SysUtils, Opengl12, sdl, sdl_Image,logger, sdlutils, parSdlMain;

const MaxMaterials=50;                                                          //Max materials that will ever be used

const FogMode : array[0..2] of GLuint = (GL_EXP,GL_EXP2,GL_LINEAR);

type GLvector3f  = array[0..2]  of glFloat;
PGLvector3f = ^GLvector3f;

Pmatrix4f = ^tmatrix4f;

type Plane=record
   a, b, c, d : GLfloat;
end;
PPlane = ^Plane;

type TVertex=record
   x,y,z : GLFloat;
end;
type PVertex  = ^TVertex;
type TPoint3f =  TVertex;
type PPoint3f = ^TPoint3f;

type TRGBRec = record
   r,g,b : byte;
end;
PRGBRec = ^TRGBRec;

type TRGBARec = record
   r,g,b,a : byte;
end;
PRGBARec = ^TRGBARec;

type TMaterial=record
   Name : string[255];
   TextureFileName : string[255];
   Surface, MaskSurface : PSDL_Surface;                                         // used for texture loading
   TextureId, MaskId : GLUint;
   TextureOffset,TextureRepeat : TVertex;                                       // uv params

   Diffuse,Specular,Ambient,Emissive : tvector4f;                              // normal color, shine color, minimum color, glow color
   Shininess : GLUint;                                                          // shine level 0-127

   ListBase : GLUInt;                                                           // used for text
   LightId : GLUint;                                                            // used for lights
   Pos : tvector4f;                                                            // used for lights
end;
PMaterial=^TMaterial;
type TLight = TMaterial;
PLight = ^TLight;

Type TFace=record
   Vertices : array[0..3] of PVertex;                                           // vert coords
   TextureVertices : array[0..3] of PVertex;                                    // tex coords
   Material : gluint;                                                           // index into material list (saves memory)
   isQuad : boolean;                                                            // 4 or 3 verts
end;
PFace=^TFace;

type TArrayByte=array[0..0] of byte;
PTArrayByte = ^TArrayByte;

type TArrayFloat=array[0..0] of GlFloat;
PTArrayFloat = ^TArrayFloat;

type Tcamera=record
   Pos,Rot,Look : Tpoint3f;
   norm : glFloat;
end;
PCamera=^TCamera;

type TFog=record
   Enabled :boolean;
   color : tvector4f;
   Filter : GLuint;
   Density : GlFloat;
   Front : GlFloat;
   Back : GlFloat;
end;
Pfog = ^TFog;

type TPARMaterial=class(tobject)
   constructor Create;
   destructor Destroy; override;
   procedure RemoveMaterial(id : gluint);
   function AddMaterial(Material:PMaterial) : glint;
   procedure SetMaterial(id: gluint);  overload;
   procedure SetMaterial(n: string); overload;
   function LoadGLTexture(matId,mipmap : GLUint) : GLUint;
   public
   NumMaterials : Gluint;
   Materials : array[0..MaxMaterials] of PMaterial;
end;


var Light0,Light1,Light2: TLight;                                               // Lighting variables
    Materials: TPARMaterial;                                                    // Material Manager object
    Camera : TCamera;                                                           // Camera Object

    gmf : array[0..255] of GLYPHMETRICSFLOAT;                                   // Glyph metrics for finding size of charactors
    PARSDLFont,PARSDLFontTGA,PARSDLFont_xyz : TMaterial;                        // font objects, material class was extended for fonts

    Fog : TFog;
    isGeforce:boolean = false;                                                  // detected later Geforce=More Power!
    PARQuadric : PGLUQuadric;

function make4fv(r,g,b,a:GLFloat):tvector4f;                                    // util function
function make3fv(r,g,b:GLFloat):GLVector3f;                                     // util function

procedure VMatMult(var M:tmatrix4f; var v:tvector4f);
procedure readstr(var f : TextFile; var s : string);

function  PARSetupFog(f : pfog) : boolean;

function LoadImage(filename : string) : PSDL_Surface;
function LoadTGA(filename : string) : PSDL_Surface;
function LoadBmp(filename : string) : PSDL_Surface;

function BuildFont(FontName : string;CharSetType:cardinal) : boolean;
function BuildFontTGA(filename:string) : boolean;
procedure KillFont;

procedure glPrint(st : string); overload;
procedure glPrint(x,y:glFloat;st : string); overload;
procedure glPrintTGA(x, y : GLint; fontset : cardinal; text : string); overload
procedure glPrintTGA(x, y, z : GLFloat; fontset : cardinal; text : string); overload;

procedure setLight(light : Plight);
procedure setCamera(camera : PCamera);

procedure SolidCube(size:GlFloat);

function DefaultInitGL : boolean;
procedure calcBillboardMatrix(x,y,z:glfloat);

implementation

procedure buildRot(theta,x, y, z: glfloat; var m : tmatrix4f);
var d,ct,st : glfloat;
begin
   ct := cos(theta*M_PI_OVER_180);
   st := sin(theta*M_PI_OVER_180);

   d := x*x + y*y + z*z;
   if (d > 0) then begin
      d := 1/d;
      x := x * d;
      y := y * d;
      z := z * d;
   end;

   m[0,0] := 1; m[0,1] := 0; m[0,2] := 0; m[0,3] := 0;
   m[1,0] := 0; m[1,1] := 1; m[1,2] := 0; m[1,3] := 0;
   m[2,0] := 0; m[2,1] := 0; m[2,2] := 1; m[2,3] := 0;
   m[3,0] := 0; m[3,1] := 0; m[3,2] := 0; m[3,3] := 1;

   m[0,0] := x*x + ct*(1-x*x) + st*0;
   m[1,0] := x*y + ct*(0-x*y) + st*-z;
   m[2,0] := x*z + ct*(0-x*z) + st*y;

   m[0,1] := y*x + ct*(0-y*x) + st*z;
   m[1,1] := y*y + ct*(1-y*y) + st*0;
   m[2,1] := y*z + ct*(0-y*z) + st*-x;

   m[0,2] := z*x + ct*(0-z*x) + st*-y;
   m[1,2] := z*y + ct*(0-z*y) + st*x;
   m[2,2] := z*z + ct*(1-z*z) + st*0;
end;

procedure calcBillboardMatrix(x,y,z:glfloat);
var mat : tmatrix4f;
begin
   glGetFloatv(GL_MODELVIEW_MATRIX, @mat);
   buildRot(180*arctan2(mat[2,0], mat[2,2])/M_PI, x, y, z, mat);
   glMultMatrixf(@mat);
end;

function make4fv(r,g,b,a:GLFloat):tvector4f;
begin
   result[0]:=r;
   result[1]:=g;
   result[2]:=b;
   result[3]:=a;
end;

function make3fv(r,g,b:GLFloat):GLVector3f;
begin
   result[0]:=r;
   result[1]:=g;
   result[2]:=b;
end;

constructor TPARMaterial.Create;
begin
   inherited create;
   NumMaterials:=0;
   zeroMemory(@Materials,sizeof(Materials));
end;

destructor TPARMaterial.Destroy;
var i : gluint;
begin
   for i:=0 to MaxMaterials-1 do begin
      if assigned(Materials[i]) then begin
         freemem(Materials[i],sizeof(TMaterial));
      end;
   end;
   inherited Destroy;
end;

procedure TPARMaterial.RemoveMaterial(id : gluint);
begin
   if assigned(materials[id]) then freemem(materials[id],sizeof(TMaterial));
   Materials[id]:=nil;
   dec(NumMaterials);
end;

function TPARMaterial.AddMaterial(Material : PMaterial) : glint;
var i : gluint;
begin
   result:=-1;
   if not assigned(Material) then begin
      Log.LogError( Format( 'nil material pointer.', [] ),   'AddMaterial' );
      exit;
   end;
   if Material.Name='' then Material.Name:=extractfilename(material.TextureFileName);
   for i:=0 to MaxMaterials-1 do begin
      if not assigned(Materials[i]) then begin
         result:=i;
         Materials[i]:=Material;
         inc(NumMaterials);
         exit;
      end;
   end;
   Log.LogError( Format( 'No material storage space for %s.', [Material.Name] ),   'AddMaterial' );
end;

procedure TPARMaterial.SetMaterial(n : string);
var i,id : longint;
begin
   id:=-1;
   for i := 0 to NumMaterials-1 do if Materials[i].Name=n then id:=i;
   if id=-1 then begin
      Log.LogError( Format( 'No material with name %s.', [n] ),   'SetMaterial' );
      exit;
   end;
   with Materials[id]^ do begin
      glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,   @Diffuse);
      glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR,  @Specular);
      glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,   @Ambient);
      glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION,  @Emissive);
      glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, @Shininess);

      glBindTexture(GL_TEXTURE_2D, TextureId);
      glColor4f(Diffuse[0],Diffuse[1],Diffuse[2],Diffuse[3]);
   end;
end;

procedure TPARMaterial.SetMaterial(id: gluint);
begin
   if not assigned(Materials[id]) then begin
      Log.LogError( Format( 'No material assigned to id %d.', [id] ),   'SetMaterial' );
      exit;
   end;

   with Materials[id]^ do begin
      glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,   @Diffuse);
      glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR,  @Specular);
      glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,   @Ambient);
      glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION,  @Emissive);
      glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, @Shininess);

      glBindTexture(GL_TEXTURE_2D, TextureId);
      glColor4f(Diffuse[0],Diffuse[1],Diffuse[2],Diffuse[3]);
   end;
end;

procedure VMatMult(var M:tmatrix4f; var v:tvector4f);
var res : tvector4f;   				        			// Hold Calculated Results
begin
   res[0]:=M[0,0]*v[0]+M[1,0]*v[1]+M[2,0]*v[2]+M[3,0]*v[3];
   res[1]:=M[0,1]*v[0]+M[1,1]*v[1]+M[2,1]*v[2]+M[3,1]*v[3];
   res[2]:=M[0,2]*v[0]+M[1,2]*v[1]+M[2,2]*v[2]+M[3,2]*v[3];
   res[3]:=M[0,3]*v[0]+M[1,3]*v[1]+M[2,3]*v[2]+M[3,3]*v[3];
   v[0]:=res[0];	        						// Results Are Stored Back In v[]
   v[1]:=res[1];
   v[2]:=res[2];
   v[3]:=res[3];		        					// Homogenous Coordinate
end;

procedure readstr(var f : textfile; var s : string);
begin
   readln(f, s);
   while ((copy(s, 1, 1) = '/') or (length(s) = 0)) do begin
      if eof(f) then exit;
      readln(f, s);
   end;
end;

function LoadImage(filename : string) : PSDL_Surface;
begin
   result:=nil;
   if (filename='') then begin
      Log.LogError( Format( 'No file name specified.', [] ),   'LoadImage' );
      exit;
   end;

   if not fileexists(filename) then begin
      Log.LogError( Format( 'Image file: %s does not exists.', [filename] ),   'LoadImage' );
      exit;
   end;

   if uppercase(copy(filename,length(filename)-2,3))=uppercase('BMP') then begin
      result:=LoadBmp(filename);
      if not assigned(result) then Log.LogError( Format( 'LoadBMP Returned nil pointer for : %s.', [filename] ),   'LoadImage' );
      exit;
   end;
   if uppercase(copy(filename,length(filename)-2,3))=uppercase('TGA') then begin
      result:=LoadTGA(filename);
      if not assigned(result) then Log.LogError( Format( 'LoadTGA Returned nil pointer for : %s.', [filename] ),   'LoadImage' );
      exit;
   end;
   Log.LogError( Format( 'Unsupported Image type for file: %s.', [filename] ),   'LoadImage' );
end;

function LoadBmp(filename : string) : PSDL_Surface;
var ts : PSDL_Surface;
    srcP : PRGBRec;
    x,y : gluint;
begin
   result:=nil;
   if (filename='') then begin
      Log.LogError( Format( 'No filename specified.', [] ),   'LoadBMP' );
      exit;
   end;
   if uppercase(copy(filename,length(filename)-2,3))<>'BMP' then begin
      Log.LogError( Format( 'Unsupported Image type for file: %s.', [filename] ),   'LoadBMP' );
      exit;
   end;
   if not fileexists(filename) then begin
      Log.LogError( Format( 'Image file: %s does not exists.', [filename] ),   'LoadBMP' );
      exit;
   end;
   ts:=IMG_LoadBMP_RW(SDL_RWFromFile(pchar(filename), 'rb'));
   if not assigned(ts) then begin
      Log.LogError( Format( 'Load BMP for file %s failed.', [filename] ),   'LoadBMP' );
      exit;
   end;
   SDL_FlipRectV(ts,nil);

   if ts.format.BitsPerPixel=24 then begin
      srcP:=ts.pixels;
      for y:=0 to ts.h - 1 do begin                                             // Loop through entire image
         for x:=0 to ts.w - 1 do begin
            srcP.r:=srcP.r xor srcP.b;                                          // swap blue and red
            srcP.b:=srcP.r xor srcP.b;
            srcP.r:=srcP.r xor srcP.b;
            inc(srcP);
         end;
      end;
   end;
   result:=ts;
end;

function LoadTGA(filename : string) : PSDL_Surface;
var ts : PSDL_Surface;
    srcPA : PRGBARec;
    srcP : PRGBRec;
    x,y : gluint;
begin
   result:=nil;

   if (filename='') then begin
      Log.LogError( Format( 'No filename specified.', [] ),   'LoadTGA' );
      exit;
   end;
   if uppercase(copy(filename,length(filename)-2,3))<>'TGA' then begin
      Log.LogError( Format( 'Unsupported Image type for file: %s.', [filename] ),   'LoadTGA' );
      exit;
   end;
   if not fileexists(filename) then begin
      Log.LogError( Format( 'Image file: %s does not exists.', [filename] ),   'LoadTGA' );
      exit;
   end;

   ts:=IMG_LoadTGA_RW(SDL_RWFromFile(pchar(filename), 'rb'));                   // load tga
   if not assigned(ts) then begin
      Log.LogError( Format( 'Load TGA for file %s failed.', [filename] ),   'LoadBMP' );
      exit;
   end;

   SDL_FlipRectV(ts,nil);                                                       // flip image verticaly
   if ts.format.BitsPerPixel=32 then begin
      srcPA:=ts.pixels;
      for y:=0 to ts.h - 1 do begin                                             // Loop through entire image
         for x:=0 to ts.w - 1 do begin
            srcPA.r:=srcPA.r xor srcPA.b;                                       // swap blue and red
            srcPA.b:=srcPA.r xor srcPA.b;
            srcPA.r:=srcPA.r xor srcPA.b;
            inc(srcPA);
         end;
      end;
   end else begin
      srcP:=ts.pixels;
      for y:=0 to ts.h - 1 do begin                                             // Loop through entire image
         for x:=0 to ts.w - 1 do begin
            srcP.r:=srcP.r xor srcP.b;                                          // swap blue and red
            srcP.b:=srcP.r xor srcP.b;
            srcP.r:=srcP.r xor srcP.b;
            inc(srcP);
         end;
      end;
   end;

   result:=ts;
end;

function TPARMaterial.LoadGLTexture(matId,mipmap : GLUint) : GLUint;
var glType : GluInt;
    x,y : gluint;

    pixel : TRGBARec;
    pixel2 : TRGBRec;

    srcP : PRGBARec;
    dstP : PRGBRec;
begin
   result:=0;
   Materials[matid].Surface:=LoadImage(Materials[matid].TextureFileName);       // load image data from file
   if not assigned(Materials[matid].Surface) then begin
      Log.LogError( Format( 'Load image for file %s failed.', [Materials[matid].TextureFileName] ),   'PARMaterial.LoadGLTexture' );
      exit;
   end;
   glGenTextures(1, @Materials[matid].TextureId); 	        	        // Generate OpenGL texture IDs
   if Materials[matid].TextureId < 1  then begin
      Log.LogError( Format( 'glGenTextures failed for  file %s.', [Materials[matid].TextureFileName] ),   'PARMaterial.LoadGLTexture' );
      exit;
   end;


   glBindTexture(GL_TEXTURE_2D, Materials[matid].TextureId);	        	// Bind Our Texture
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);     	// Linear Filtered
   if mipmap=0 then glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)     	// Linear Filtered
      else glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);


   gltype := GL_RGB;
   if Materials[matid].Surface.format.BitsPerPixel = 24 then gltype := GL_RGB;  // Was The TGA 24 Bits  If So Set The 'type' To GL_RGB
   if Materials[matid].Surface.format.BitsPerPixel = 32 then gltype := GL_RGBA;	// Was The TGA 32 Bits  If So Set The 'type' To GL_RGBA

   if mipmap=0 then glTexImage2D(GL_TEXTURE_2D, 0, gltype, Materials[matid].Surface.w, Materials[matid].Surface.h, 0, gltype, GL_UNSIGNED_BYTE, Materials[matid].Surface.pixels)
      else gluBuild2DMipmaps(GL_TEXTURE_2D, gltype, Materials[matid].Surface.w, Materials[matid].Surface.h, gltype, GL_UNSIGNED_BYTE, Materials[matid].Surface.pixels);

//	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
//	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

   result:=Materials[matid].TextureId;

   if Materials[matid].Surface.format.BitsPerPixel = 32 then begin
      glGenTextures(1, @Materials[matid].MaskId);	        		// Generate OpenGL texture IDs

      if Materials[matid].MaskId<1  then begin
         Log.LogError( Format( 'glGenTextures Mask failed for  file %s.', [Materials[matid].TextureFileName] ),   'PARMaterial.LoadGLTexture' );
         exit;
      end;

      Materials[matid].MaskSurface:=SDL_CreateRGBSurface(Materials[matid].Surface.flags,Materials[matid].Surface.w,Materials[matid].Surface.h,24,Materials[matid].Surface.format.RMask,Materials[matid].Surface.format.GMask,Materials[matid].Surface.format.BMask,Materials[matid].Surface.format.AMask);
      srcP:=Materials[matid].Surface.pixels;
      dstP:=Materials[matid].MaskSurface.pixels;
      for y:=0 to Materials[matid].Surface.h - 1 do begin
         for x:=0 to Materials[matid].Surface.w - 1 do begin
            Pixel:=SrcP^;

            pixel2.r:=pixel.a;
            pixel2.g:=pixel.a;
            pixel2.b:=pixel.a;

            dstP^:=pixel2;
            inc(dstP);
            inc(srcP);
         end;
      end;
      gltype := GL_RGB;	                                			// RGB bacuse no alpha
      glBindTexture(GL_TEXTURE_2D, Materials[matid].MaskId);		        // Bind Our Texture
      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);     	// Linear Filtered
      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);     	// Linear Filtered
      glTexImage2D(GL_TEXTURE_2D, 0, gltype, Materials[matid].MaskSurface.w, Materials[matid].MaskSurface.h, 0, gltype, GL_UNSIGNED_BYTE, Materials[matid].MaskSurface.pixels);
   end;
end;

// Always check for extension-availability during run-time! Here we go!
function isInString(extensions: PChar; const extension: String): Boolean;
var start: PChar;
    where, terminator: PChar;
begin
   // Extension names should not have spaces. or be empty
   if (Pos(' ', extension) <> 0) or (extension = '') then begin
      Result := FALSE;
      Exit;
   end;

  { It takes a bit of care to be fool-proof about parsing the OpenGL extensions
    string. Don't be fooled by sub-strings, etc. }
   start := extensions;
   while TRUE do begin
      where := StrPos(start, PChar(extension));
      if where = nil then Continue;
      terminator := Pointer(Integer(where) + Length(extension));
      if (where = start) or (PChar(Integer(where) - 1)^ = ' ') then begin
         if (terminator^ = ' ') or (terminator^ = #0) then begin
            Result := TRUE;
            Exit;
         end;
      end;
      start := terminator;
   end;
   Result := FALSE;
end;

function BuildFont(FontName : string;CharSetType:cardinal) : boolean;
var font: HFONT;                                        	                // Windows Font ID
begin
   PARSDLFont.ListBase := glGenLists(96);       	                        // Storage For 96 Characters
   Font := CreateFont(-24,			                        	// Height Of Font
        	     0,				                         	// Width Of Font
    	             0,				                        	// Angle Of Escapement
		     0,				                         	// Orientation Angle
		     FW_BOLD,	                        	                // Font Weight
		     0,			       	                                // Italic
		     0,		                	       	                // Underline
		     0,			       	                                // Strikeout
		     CharSetType,	                	                // Character Set Identifier
		     OUT_TT_PRECIS,	                                        // Output Precision
		     CLIP_DEFAULT_PRECIS,	                                // Clipping Precision
		     ANTIALIASED_QUALITY,	                           	// Output Quality
		     FF_DONTCARE or DEFAULT_PITCH,                       	// Family And Pitch
		     PCHAR(FontName));		       	                        // Font Name

   SelectObject(wglGetCurrentDC, Font);	       	                                // Selects The Font We Want
   wglUseFontBitmaps(wglGetCurrentDC, 32, 96, PARSDLFont.ListBase);             // Builds 96 Characters Starting At Character 32
   result:=true;
end;

procedure glPrint(st : string);
begin
   if (st = '') or (PARSDLFont.ListBase=0)then Exit;       	        	// If There's No Text
   glPushAttrib(GL_ALL_ATTRIB_BITS);
      glPushMatrix;
      glLoadIdentity;
      gltranslatef(-0.35,0.2,-1);
      glDisable(GL_TEXTURE_GEN_S);  	        		                // Auto Texture Generation
      glDisable(GL_TEXTURE_GEN_T);	                     	                // Auto Texture Generation
      glDisable(GL_Fog);
      glDisable(GL_Lighting);
      glDisable(GL_Blend);
      glDisable(GL_Depth_test);
      glEnable(GL_Texture_2d);
      
      glRasterPos2f(0,0);                                                       // Top Left Of Raster

      glListBase(PARSDLFont.ListBase - 32);	     				// Sets The Base Character to 32
      glCallLists(length(st), GL_UNSIGNED_BYTE, @st[1]);	                // Draws The Display List Text
      glPopMatrix;
   glPopAttrib();								// Pops The Display List Bits
end;

procedure glPrint(x,y:glFloat;st : string);
begin
   if (st = '') or (PARSDLFont.ListBase=0)then Exit;   	        		// If There's No Text
   glPushAttrib(GL_ALL_ATTRIB_BITS);
      glPushMatrix;
      glLoadIdentity;
      gltranslatef(x,y,-1);
      glDisable(GL_TEXTURE_GEN_S);  	        		                // Auto Texture Generation
      glDisable(GL_TEXTURE_GEN_T);        	        	                // Auto Texture Generation
      glDisable(GL_Fog);
      glDisable(GL_Lighting);
      glDisable(GL_Blend);
      glDisable(GL_Depth_test);
      glEnable(GL_Texture_2d);
      
      glRasterPos2f(0,0);                                                       // Top Left Of Raster

      glListBase(PARSDLFont.ListBase - 32);					// Sets The Base Character to 32
      glCallLists(length(st), GL_UNSIGNED_BYTE, @st[1]);	                // Draws The Display List Text
      glPopMatrix;
   glPopAttrib();								// Pops The Display List Bits
end;

function BuildFontTGA(filename:string) : boolean;
var loop1,gltype : cardinal;
    cx, cy : single;
begin
   result:=false;
   if not fileexists(filename) then exit;

   glPushAttrib(GL_ALL_ATTRIB_BITS);

   PARSDLFontTGA.Surface:=LoadTGA(filename);

   glGenTextures(1, @PARSDLFontTGA.TextureId);
   glBindTexture(GL_TEXTURE_2D,PARSDLFontTGA.TextureId );


   gltype := GL_RGB;
   if PARSDLFontTGA.surface.format.BitsPerPixel = 24 then gltype := GL_RGB;	// Was The TGA 24 Bits  If So Set The 'type' To GL_RGB
   if PARSDLFontTGA.surface.format.BitsPerPixel = 32 then gltype := GL_RGBA;	// Was The TGA 32 Bits  If So Set The 'type' To GL_RGBA


   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
   glTexImage2D(GL_TEXTURE_2D, 0, gltype, PARSDLFontTGA.surface.w, PARSDLFontTGA.surface.h, 0, gltype, GL_UNSIGNED_BYTE, PARSDLFontTGA.surface^.pixels);


   PARSDLFontTGA.ListBase := glGenLists(256);
   PARSDLFont_xyz.ListBase := glGenLists(256);
   glBindTexture(GL_TEXTURE_2D, PARSDLFontTGA.TextureId);
   for loop1 := 0 to 255 do begin
      cx := (loop1 mod 16) / 16.0;
      cy := (loop1 div 16) / 16.0;

      glNewList(PARSDLFontTGA.ListBase+loop1,GL_COMPILE);
         glBegin(GL_QUADS);
            glTexCoord2f(cx, 1.0 - cy - 0.0625);
            glVertex2d( 0,16);
            glTexCoord2f(cx + 0.0625, 1.0 - cy - 0.0625);
            glVertex2i(16,16);
            glTexCoord2f(cx + 0.0625, 1.0 - cy);
            glVertex2i(16, 0);
            glTexCoord2f(cx, 1.0 - cy);
            glVertex2i( 0, 0);
         glEnd;
         glTranslated(14,0,0);
      glEndList;

      glNewList(PARSDLFont_xyz.ListBase+loop1,GL_COMPILE);
         glBegin(GL_QUADS);
            glTexCoord2f(cx, 1.0 - cy - 0.0625);
            glVertex3f( 0,0.0625,0);
            glTexCoord2f(cx + 0.0625, 1.0 - cy - 0.0625);
            glVertex3f(0.0625,0.0625,0);
            glTexCoord2f(cx + 0.0625, 1.0 - cy);
            glVertex3f(0.0625, 0,0);
            glTexCoord2f(cx, 1.0 - cy);
            glVertex3f( 0, 0,0);
         glEnd;
         glTranslatef(0.0625,0,0);
      glEndList;


   end;
   SDL_FreeSurface(PARSDLFont.Surface);
   PARSDLFontTGA.Diffuse:=make4fV(1,0,0,1);
   PARSDLFont_xyz.Diffuse:=make4fV(1,0,0,1);
   glPopAttrib();
   result:=true;
end;


procedure KillFont;
begin
   if PARSDLFont.ListBase <> 0 then glDeleteLists(PARSDLFont.ListBase, 96);	     // Delete All 96 Characters
   if PARSDLFontTGA.ListBase <> 0 then glDeleteLists(PARSDLFontTGA.ListBase, 256);   // Delete All 256 Characters
   if PARSDLFont_xyz.ListBase <> 0 then glDeleteLists(PARSDLFont_xyz.ListBase, 256); // Delete All 256 Characters
end;

procedure glPrintTGA(x, y : GLint; fontset : cardinal; text : string);
begin
   if PARSDLFont.ListBase = 0 then exit;                                        // Have not initalized our font

   if text='' then exit;                                                        // No text, no print

   if (fontset > 1) then fontset := 1;                                          // We only have one font set

   glPushAttrib(GL_ALL_ATTRIB_BITS);
   glPushMatrix();                                                              //Push Model Matrix

   glDisable(GL_TEXTURE_GEN_S);  	        		                // Auto Texture Generation
   glDisable(GL_TEXTURE_GEN_T);	        	        	                // Auto Texture Generation
   glDisable(Gl_LIGHTING);
   glEnable(GL_BLEND);								// Enable Blending
   glBlendFunc(GL_ONE, GL_ONE);                                                 // Copy all but black
   glenable(GL_TEXTURE_2D);

   glMatrixMode(GL_PROJECTION);                                                 // Select The Projection Matrix
   glPushMatrix();                                                              // Push Projection Matrix

   glLoadIdentity;                                                              // Reset The Modelview Matrix
   glOrtho(0.0,PAR_System.Screen.Width,PAR_System.Screen.Height,0.0,-1.0,1.0);  // Set 2D Mode
   glMatrixMode(GL_MODELVIEW);                                                  // Select The Modelview Matrix
   glLoadIdentity;                                                              // Reset The Modelview Matrix

   glBindTexture(GL_TEXTURE_2D, PARSDLFont.TextureId);                          // Bind our font texture
   glTranslated(x,y,1);                                                         // move to proper coords
   glListBase(PARSDLFontTGA.ListBase-32+(128*fontset));                         // Select correct polly list
   glScalef(1,2,1);                                                             // Scale texture to proper perportion

   glColor4fV(@PARSDLFontTGA.Diffuse);
   glCallLists(length(text),GL_UNSIGNED_BYTE, @text[1]);                        // Call polly list

   glMatrixMode(GL_PROJECTION);                                                 // Select The Projection Matrix
   glPopMatrix();                                                               // Pop Projection Matrix

   glMatrixMode(GL_MODELVIEW);                                                  // Select The Modelview Matrix
   glPopMatrix();                                                               // Pop Model Matrix
   glPopAttrib();
end;

procedure glPrintTGA(x, y, z : GLFloat; fontset : cardinal; text : string);
begin
   if PARSDLFont_xyz.ListBase = 0 then exit;                                    // Have not initalized our font

   if text='' then exit;                                                        // No text, no print

   if (fontset > 1) then fontset := 1;                                          // We only have two font sets

   glPushAttrib(GL_ALL_ATTRIB_BITS);
   glDisable(GL_TEXTURE_GEN_S);  	        		                // Auto Texture Generation
   glDisable(GL_TEXTURE_GEN_T);	        	        	                // Auto Texture Generation
   glDisable(Gl_LIGHTING);
   glDisable(GL_BLEND);								// Enable Blending
   glBlendFunc(GL_ONE, GL_ONE);                                                 // Copy all but black
   glenable(GL_TEXTURE_2D);
   glBindTexture(GL_TEXTURE_2D, PARSDLFontTGA.TextureId);                       // Bind our font texture
   glColor4fV(@PARSDLFont_xyz.Diffuse);
   glMatrixMode(GL_MODELVIEW);                                                  // Select The Modelview Matrix
   glPushMatrix();                                                              //Push Model Matrix
      glTranslatef(x,y,z);                                                      // move to proper coords
      glRotatef(90,0,1,0);
      calcBillBoardMatrix(1,0,0);
      glRotatef(-90,0,0,1);
      glRotatef(180,0,1,0);

      glListBase(PARSDLFont_xyz.ListBase-32+(128*fontset));                     // Select correct polly list
      glCallLists(length(text),GL_UNSIGNED_BYTE, @text[1]);                     // Call polly list

   glPopMatrix();                                                               // Pop Model Matrix
   glPopAttrib();
end;

procedure setLight(light : plight);
begin
   glLightfv(Light.LightId, GL_POSITION, @Light^.Pos);
   glLightfv(Light.LightId, GL_AMBIENT,  @Light^.Ambient);
   glLightfv(Light.LightId, GL_DIFFUSE,  @Light^.Diffuse);
   glLightfv(Light.LightId, GL_SPECULAR, @Light^.Specular);
end;

procedure setCamera(camera : PCamera);
begin
   gluLookAt(-camera.pos.x,camera.pos.y,-camera.pos.z,camera.look.x, camera.look.y, camera.look.z,0,1,0);
end;

function  PARSetupFog(f : pfog) : boolean;
begin
   glFogi(GL_FOG_MODE, fogMode[Fog.Filter]);	                        	// Fog Mode
   glFogfv(GL_FOG_COLOR, @Fog.Color);			                        // Set Fog Color
   glFogf(GL_FOG_DENSITY, Fog.Density); 	                        	// How Dense Will The Fog Be
   glHint(GL_FOG_HINT, GL_NICEST);			                        // Fog Hint Value
   glFogf(GL_FOG_START, Fog.Front); 		                        	// Fog Start Depth
   glFogf(GL_FOG_END, Fog.Back);    		                        	// Fog End Depth
   glEnable(GL_FOG);				                        	// Enables GL_FOG
   glClearColor(Fog.Color[0], Fog.Color[1], Fog.Color[2], Fog.Color[3]);        // fog color Background
   result:=true;
end;

procedure InitMultiTexture;
begin
   PAR_System.Screen.CanMultiTexture:=true;
   glActiveTextureARB := wglGetProcAddress('glActiveTextureARB');

   if not assigned(glActiveTextureARB) then begin
      Log.LogError( Format( 'Error: can not load multi-texture extension glActiveTextureARB', [] ),   'IntMultiTexture' );
      PAR_System.Screen.CanMultiTexture:=false;
      exit;
   end;

   glMultiTexCoord2fARB := wglGetProcAddress('glMultiTexCoord2fARB');

   if not assigned(glMultiTexCoord2fARB) then begin
      Log.LogError( Format( 'Error: can not load multi-texture extension glMultiTexCoord2fARB', [] ),   'IntMultiTexture' );
      PAR_System.Screen.CanMultiTexture:=false;
      exit;
   end;
end;

function DefaultInitGL : boolean;
begin
   result:=true;

   Fog.Enabled:=false;
   Fog.Filter:=GL_LINEAR;
   Fog.Color[0]:=0.5;
   Fog.Color[1]:=0.5;
   Fog.Color[2]:=0.5;
   Fog.Color[3]:=1.0;
   Fog.Density:=0.1;
   Fog.front:=5;
   Fog.back:=10;

   Light0.LightId:=GL_Light0;
   Light0.Pos[0] := 50;
   Light0.Pos[1] := 50;
   Light0.Pos[2] := 50;
   Light0.Pos[3] := 0;

   Light0.Diffuse[0] := 1.0;
   Light0.Diffuse[1] := 1.0;
   Light0.Diffuse[2] := 1.0;
   Light0.Diffuse[3] := 1.0;

   Light0.Specular[0] := 1.0;
   Light0.Specular[1] := 1.0;
   Light0.Specular[2] := 1.0;
   Light0.Specular[3] := 1.0;

   Light0.Ambient[0] := 0.5;
   Light0.Ambient[1] := 0.5;
   Light0.Ambient[2] := 0.5;
   Light0.Ambient[3] := 1.0;

   Camera.Pos.x:=0;
   Camera.Pos.y:=0;
   Camera.Pos.z:=-10;

   Camera.Look.x:=0;
   Camera.Look.y:=0;
   Camera.Look.z:=0;

   Camera.Rot.x:=0;
   Camera.Rot.y:=0;
   Camera.Rot.z:=0;


   glClearColor(0.0, 0.0, 0.0, 0.0);                                            // Clear The Background Color To Black
   glClearDepth(1.0);			                                        // Enables Clearing Of The Depth Buffer
   glClearStencil(0);								// Clear The Stencil Buffer To 0
   glShadeModel(GL_SMOOTH);     	                                        // Enables Smooth Color Shading
   glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_FASTEST);                           // Realy Nice perspective calculations
   glEnable(GL_DEPTH_TEST);     	                                        // Enable Depth Testing
   glDepthFunc(GL_LEQUAL);  			                                // The Type Of Depth Test To Do
   glEnable(GL_TEXTURE_2D);	                                                // Enable 2D Texture Mapping
   glEnable(GL_AUTO_NORMAL);                                                    // Auto create normals
   glEnable(GL_NORMALIZE);                                                      // Auto Convert All Normals to Unit Length
   glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );                                 // Both sides of polygons are solod
   glDisable(GL_CULL_FACE);                                                      // Remove back face

   glHint(GL_POLYGON_SMOOTH_HINT,GL_FASTEST);                                   // aa and quality settings
//   glEnable(GL_POLYGON_SMOOTH);
   glHint(GL_LINE_SMOOTH_HINT,GL_FASTEST);
//   glEnable(GL_LINE_SMOOTH);
   glHint(GL_POINT_SMOOTH_HINT,GL_FASTEST);
//   glEnable(GL_POINT_SMOOTH);

   if PAR_System.Screen.BPP=16 then glEnable(GL_DITHER);                        // looks better on 16 bit cards
   glEnable(GL_LIGHTING);                                                       // Let their be light
   glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, ORD(GL_FALSE));                       // Light both sides of poly's

   SetLight(@light0);
   glEnable(Light0.LightId);

   if (Fog.Enabled) then PARSetupFog(@fog);
   isGeforce:=(pos('GEFORCE',uppercase(glGetString(GL_RENDERER)))>0);
   Materials:=TPARMaterial.Create;
   InitMultiTexture;

   PARQuadric:=gluNewQuadric();			                                        // Create A Pointer To The Quadric Object
   gluQuadricNormals(PARQuadric, GLU_SMOOTH);	                                        // Create Smooth Normals
   gluQuadricTexture(PARQuadric, true);                                       	        // Create Texture Coords

end;

procedure SolidCube(size:GlFloat);
begin
   glBegin(GL_QUADS);
      glNormal3f (0.0, 0.0, size);
      glTexCoord2f(0,0);
      glVertex3f (size, size, size);
      glTexCoord2f(1,0);
      glVertex3f (-size, size, size);
      glTexCoord2f(1,1);
      glVertex3f (-size, -size, size);
      glTexCoord2f(0,1);
      glVertex3f (size, -size, size);
   glEnd;

   glBegin(GL_QUADS);
      glNormal3f (0.0, 0.0, -size);
      glTexCoord2f(0,0);
      glVertex3f (size, size, -size);
      glTexCoord2f(1,0);
      glVertex3f (size, -size, -size);
      glTexCoord2f(1,1);
      glVertex3f (-size, -size, -size);
      glTexCoord2f(0,1);
      glVertex3f (-size, size, -size);
   glEnd;

   glBegin(GL_QUADS);
      glNormal3f (-size, 0.0, 0.0);
      glTexCoord2f(0,0);
      glVertex3f (-size, size, size);
      glTexCoord2f(1,0);
      glVertex3f (-size, size, -size);
      glTexCoord2f(1,1);
      glVertex3f (-size, -size, -size);
      glTexCoord2f(0,1);
      glVertex3f (-size, -size, size);
   glEnd;

   glBegin(GL_QUADS);
      glNormal3f (size, 0.0, 0.0);
      glTexCoord2f(0,0);
      glVertex3f (size, size, size);
      glTexCoord2f(1,0);
      glVertex3f (size, -size, size);
      glTexCoord2f(1,1);
      glVertex3f (size, -size, -size);
      glTexCoord2f(0,1);
      glVertex3f (size, size, -size);
   glEnd;

   glBegin(GL_QUADS);
      glNormal3f (0.0, size, 0.0);
      glTexCoord2f(0,0);
      glVertex3f (-size, size, -size);
      glTexCoord2f(1,0);
      glVertex3f (-size, size, size);
      glTexCoord2f(1,1);
      glVertex3f (size, size, size);
      glTexCoord2f(0,1);
      glVertex3f (size, size, -size);
   glEnd;

   glBegin(GL_QUADS);
      glNormal3f (0.0, -size, 0.0);
      glTexCoord2f(0,0);
      glVertex3f (-size, -size, -size);
      glTexCoord2f(1,0);
      glVertex3f (size, -size, -size);
      glTexCoord2f(1,1);
      glVertex3f (size, -size, size);
      glTexCoord2f(0,1);
      glVertex3f (-size, -size, size);
   glEnd;
end;


initialization

finalization
   if assigned(Materials) then Materials.Destroy;

end.

