unit ParSdlMain;
// This file is part of the PAR SDL Framework
// Created by Paul Robello by examples and demos from but not limited to:
// nehe.gamedev.net, www.delphi-jedi.org, romka.demonews.com, www.nutty.org.
// Most code has been heavily modified for speed and integration.
// PAR SDL Framework is an always changing project that may not be backward
// compatable with other projects.
// Questions, comments, ravs, flames email paulr@cal.net

interface

uses windows, SysUtils, dialogs, mmsystem, sdl, Logger, OpenGL12, sfont;

type TPARtimer = record
   frequency : int64;
   resolution : single;
   mm_timer_start : longword;
   mm_timer_elapsed : longword;
   performance_timer : boolean;

   performance_timer_start : int64;
   performance_timer_elapsed : int64;
end;


type TAppStatus = record							// We Use A Struct To Hold Application Runtime Data
   Visible : boolean;								// Is The Application Visible? Or Iconified?
   MouseFocus : boolean;							// Is The Mouse Cursor In The Application Field?
   KeyboardFocus : boolean;							// Is The Input Focus On Our Application?
end;

type TMouse = record                                                            // Holds all relevent mouse info
   x,y   : sint16;                                                              // mouse coords
   rx,ry : sint16;                                                              // relative mouse coords
   buttons : array[1..3] of boolean;                                            // buttons left to right
end;

type TParUpdate =       procedure;                                       	// Perform Motion Updates Here
     TParEventHandle =  function : boolean;                                     // Peek and handle events
     TParDraw =         procedure;                                              // Draw Scene
     TParInitGL =       function : boolean;             			// Any OpenGL Initialization Code Goes Here													// Our Drawing Code
     TPARInitialize =   function : boolean; 					// Any Application & User Initialization Code Goes Here
     TParDeinitialize = procedure;						// Any User Deinitialization Goes Here
     TPARKeyUp =        procedure(key:TSDLKey);                                 // Key up event
     TPARKeyDown =      procedure(key:TSDLKey);                                 // Key down event
     TPARButtonUp =     procedure(button:uint8;x,y : uint16);                   // Key up event
     TPARButtonDown =   procedure(button:uint8;x,y : uint16);                   // Key down event

type TScreen = record
   Width : Uint32;      	        					// Screen Width Of Our App
   Height : Uint32;     		        				// SCreen Height Of Our App
   FOV : Uint32;
   BPP : Uint32;				        			// Screen Depth Of Our App
   Fullscreen : Boolean;                                                        // Force Full Screen
   Fullscreen_ASK : Boolean;                                                    // if not full screen, ask if want to be
   Videoflags :   Uint32; 							// Our Video Flags
   VideoInfo : PSDL_VideoInfo;                                                  // current video info and capabilites
   Surface : PSDL_Surface;                                                      // Pointer to screen surface
   PollyFill : UInt16;
   CanMultiTexture : boolean;
   bStencil : boolean;
end;

type TSDL_System = class(TObject)

   App_Name  : string;                                                          // The App Name And Caption
   E         : TSDL_Event;   							// And Event Used In The Polling Process
   Keys      : PKeyStateArr; 						        // A Pointer To An Array That Will Contain The Keyboard Snapshot
   Mouse     : TMouse;                                                          // Track Mouse actions
   AppStatus : TAppStatus;

   doFPS : boolean;                                                             // do we want FPS to calculated
   doLog : boolean;                                                             // To log or not to Log
   doGL  : boolean;                                                             // is this an OpenGL app
   doFont: boolean;                                                             // 2d Font no gl
   doGLFont : Boolean;                                                          // 2d Font with gl
   doPause : boolean;                                                           // pause game
   isProgramLooping : boolean;	        					// We're Using This One To Know If The Program Must Go On In The Main Loop
   Screen : TScreen;

   Font : PSDL_Surface;

   FPS_Count,FPS_Last_Count: uint32;
   LoopStart    : uint32;
   FPS_String   : string;
   PARTimer : TPARtimer;                                                        // High res timer
   NextTimer : single;
   LastTimer : single;
   ElapsedTimer : single;

   ParUpdate       : TParUpdate;                                                // Procedureal pointers for custom functions
   ParEventHandle  : TParEventHandle;
   ParDraw         : TParDraw;
   ParInitGL       : TParinitGL;
   ParInitialize   : TParInitialize;
   ParDeinitialize : TParDeinitialize;
   PARKeyUp        : TParKeyUp;
   PARKeyDown      : TParKeyDown;
   PARButtonUp     : TParButtonUp;
   PARButtonDown   : TParButtonDown;

   COS_LOOK, SIN_LOOK : array[0..360] of double;                                // speed up trig functions

   constructor create;
   destructor free;

   function WinMain(hInstance: HINST; hPrevInstance: HINST; pCmdLine: PChar; nCmdShow: integer): integer; stdcall;
   procedure TerminateApplication();
   procedure ToggleFullscreen();
   function InitTimers: boolean;				                // This Is Used To Init All The Timers In Our Application
   procedure ReshapeGL;                 					// Reshape The Window When It's Moved Or Resized
   function CreateWindowGL : boolean;                                        	// This Code Creates Our OpenGL Window
   procedure Handle_SDL_Event;
   function SetupFont : boolean;
   function SetupGLFont : boolean;
   function TimerGetTime : single;
   procedure init_cos_sin;
end;

function ArcTan2(const Y, X: Extended): Extended;

const M_PI = 3.141592654;
      M_ONE_80_OVER_PI = 180 / M_PI;                                            // speeds up rad to deg convert
      M_PI_OVER_180 = M_PI / 180;                                               // speeds up deg to rad convert

var PAR_System : TSDL_System;

implementation

uses math,parsdlgl;

function ArcTan2(const Y, X: Extended): Extended;
asm
   FLD     Y
   FLD     X
   FPATAN
   FWAIT
end;

procedure TSDL_System.init_cos_sin;
var i : longint;
begin
   for i:=0 to 360 do begin
      COS_LOOK[i]:=cos(i * M_PI_OVER_180);
      SIN_LOOK[i]:=sin(i * M_PI_OVER_180);
   end;
end;

function TSDL_System.TimerGetTime : single;
var time : int64;
begin
  if (PARTimer.performance_timer) then begin
      QueryPerformanceCounter(time);
      result := (time - PARTimer.performance_timer_start) * PARTimer.resolution * 1000.0;
   end else begin
      result := (timeGetTime - PARTimer.mm_timer_start) * PARTimer.resolution * 1000.0;
   end;
end;

function TSDL_System.SetupFont : boolean;
begin
   Font := SDL_LoadBMP( 'Images/24P_Copperplate_Blue.bmp' );
   Font := SDL_DisplayFormat( Font );
   InitFont( Font );
   result:=true;
end;

function TSDL_System.SetupGLFont : boolean;
begin
   result:=BuildFontTGA('font.tga') and BuildFont('Times New Roman (Western)',ANSI_CHARSET);
end;

constructor TSDL_System.create;
begin
   inherited create;
   App_Name := 'PAR SDL OpenGL';                                                // The App Name And Caption

   zeromemory(@screen, sizeof(screen));
   with screen do begin
      Width := 1024;	        		        			// Screen Width Of Our App
      Height :=   768;		        		        		// SCreen Height Of Our App
      BPP := 32;			               		        	// Screen Depth Of Our App
      FOV := 30;
      Fullscreen := FALSE;                                                      // Force Full Screen
      Fullscreen_ASK := TRUE;                                                   // if not full screen, ask if want to be
      Videoflags := 0;                            				// Our Video Flags
      VideoInfo := nil;;                                                        // describes video mode and surface
      Surface := nil;                                                           // pointer to screen surface
      bStencil := false;                                                        // to stenceil or not to stencil
      Screen.PollyFill:=GL_FILL;                                                // Start solid
   end;

   AppStatus.Visible	   := true;     	   				// At The Beginning, Our App Is Visible
   AppStatus.MouseFocus    := true;	           				// And Have Both Mouse
   AppStatus.KeyboardFocus := true;	   	        			// And Input Focus


   Keys:=nil;                                                                   // Current keyboard state
   zeromemory(@mouse, sizeof(mouse));                                           // Current mouse state

   doLog := true;                                                               // To log or not to log
   doGL  := true;                                                               // is an open gl app?
   doFPS := true;                                                               // calc and show fps?
   doFont:= false;                                                              // normal font (not for gl)
   doGLFont := true;                                                            // init opengl fonts
   doPause := false;                                                            // pause app?

   FPS_Count := 0;                                                              // Current FPS
   FPS_Last_Count := 0;                                                         // Last FPS
   NextTimer := 0;
   LoopStart := 0;                                                              // Time Main loop started

   Font :=nil;

   ParUpdate       :=nil;                                                       // User Defined functions
   ParEventHandle  :=nil;
   ParDraw         :=nil;
   ParInitGL       :=nil;
   ParInitialize   :=nil;
   ParDeinitialize :=nil;
   PARKeyUp        :=nil;
   PARKeyDown      :=nil;
   PARButtonUp     :=nil;
   PARButtonDown   :=nil;

   Init_cos_sin;
end;

destructor TSDL_System.free;
begin
   inherited destroy;
end;

function TSDL_System.InitTimers : boolean;			// This Is Used To Init All The Timers In Our Application
begin
   result:=true;
   if not(QueryPerformanceFrequency(PARtimer.frequency)) then begin
      PARTimer.performance_timer := FALSE;
      PARTimer.mm_timer_start := timeGetTime;
      PARTimer.resolution := 1.0/1000.0;
      PARTimer.frequency := 1000;
      PARTimer.mm_timer_elapsed := PARTimer.mm_timer_start;
   end else begin
      QueryPerformanceCounter(PARTimer.performance_timer_start);
      PARTimer.performance_timer := TRUE;
      PARTimer.resolution := 1.0/PARTimer.frequency;
      PARTimer.performance_timer_elapsed := PARTimer.performance_timer_start;
   end;

   NextTimer:=TimerGetTime+1000;
   LastTimer:=NextTimer-1000;
   ElapsedTimer:=1000;
end;

procedure TSDL_System.TerminateApplication();		  			// Terminate The Application
begin
   isProgramLooping:=false;
   if assigned(ParDeinitialize) then ParDeinitialize;
   SDL_QUIT;
   if doGL then UnLoadOpenGL;
   Halt(0);
end;

procedure TSDL_System.ReshapeGL;  			                        // Reshape The Window When It's Moved Or Resized
begin
   with screen do begin
      glViewport(0,0,Width,Height);						// Reset The Current Viewport
      glMatrixMode(GL_PROJECTION);					        // Select The Projection Matrix
      glLoadIdentity();								// Reset The Projection Matrix */
      if (Height<1) then Height:=1;
      gluPerspective(fov,width/height,0.1,1000.0);	                        // Calculate The Aspect Ratio Of The Window
      glMatrixMode(GL_MODELVIEW);						// Select The Modelview Matrix
      glLoadIdentity();								// Reset The Modelview Matrix
   end;
end;

function TSDL_System.CreateWindowGL : boolean;                                  // This Code Creates Our OpenGL Window
begin
   with screen do begin
      if assigned(Surface) then begin
         SDL_FreeSurface(Surface);
      end;
      Surface := SDL_SetVideoMode(Width, Height, BPP, VideoFlags);
      if(not assigned(Surface)) then begin					// We're Using SDL_SetVideoMode To Create The Window
         result:= false;							// If It Fails, We're Returning False
         exit;
      end;
      ReshapeGL;        						        // We're Calling Reshape As The Window Is Created
   end;
   result:=true;
end;

procedure TSDL_System.Handle_SDL_Event;
begin
   if assigned(ParEventHandle) then if ParEventHandle then exit;

      case E.type_ of 								// And Processing If
         SDL_QUITEV: begin							// It's a QUIT Event?
            isProgramLooping := false;						// If Yes, Make The Program Stop Looping
         end;
         SDL_VIDEORESIZE: begin							// It's a RESIZE Event?
            Screen.Width:=E.resize.w;
            Screen.Height:=E.resize.h;
            if doGL then ReshapeGL;            				        // If Yes, Recalculate The OpenGL Scene Data For The New Window
         end;
         SDL_ACTIVEEVENT: begin						      	// It's an ACTIVE Event?
            if( (E.active.state and SDL_APPACTIVE)>0 ) then begin	      	// Activity Level Changed? (IE: Iconified?)
               if( E.active.gain <>0 ) then AppStatus.Visible := true	      	// If Yes, Set AppStatus.Visible 							// Activity's Been Gained?
                  else AppStatus.Visible := false;			      	// Reset AppStatus.Visible												// Otherwhise
               end;
               if( (E.active.state and SDL_APPMOUSEFOCUS) >0 ) then begin     	// The Mouse Cursor Has Left/Entered The Window Space?
                  if( E.active.gain <>0 ) then AppStatus.MouseFocus := true   	// Report It Setting AppStatus.MouseFocus									// Entered?
                     else AppStatus.MouseFocus := false;			// The Cursor Has Left, Reset AppStatus.MouseFocus									// Otherwhise
               end;
               if((E.active.state and SDL_APPINPUTFOCUS) >0) then begin	      	// The Window Has Gained/Lost Input Focus?
                  if(E.active.gain <>0) then AppStatus.KeyboardFocus := true  	// Report It Where You Know (You Always Report, You're A Spy, Aren't You?!)									// Gained?
                     else AppStatus.KeyboardFocus := false;		    	// Reset AppStatus.KeyboardFocus												// Otherwhise
               end;
            end;

         SDL_KEYDOWN: begin							// Someone Has Pressed A Key?
            Keys := PKeyStateArr(SDL_GetKeyState(nil));		        	// Is It's So, Take A SnapShot Of The Keyboard For The Update() Func To Usee
            if assigned(PARKeyDown) then PARKeyDown(e.key.keysym.sym);
         end;
         SDL_KEYUP: begin							// Someone Has Pressed A Key?
            Keys := PKeyStateArr(SDL_GetKeyState(nil));		        	// Is It's So, Take A SnapShot Of The Keyboard For The Update() Func To Usee
            if assigned(PARKeyUp) then PARKeyUp(e.key.keysym.sym);
         end;
         SDL_MOUSEMOTION: begin
            mouse.x:=e.motion.x;
            mouse.y:=e.motion.y;
            mouse.rx:=e.motion.xrel;
            mouse.ry:=e.motion.yrel;
         end;
         SDL_MOUSEBUTTONDOWN: begin
            mouse.buttons[e.button.state]:=boolean(e.button.button);
            if assigned(PARButtonDown) then PARButtonDown(e.button.state,e.button.x,e.button.y);
         end;
         SDL_MOUSEBUTTONUP: begin
            mouse.buttons[e.button.state]:=boolean(e.button.button);
            if assigned(PARButtonUp) then PARButtonUp(e.button.state,e.button.x,e.button.y);
         end;

         SDL_SYSWMEVENT: begin
         end;
      end;

end;

procedure TSDL_System.ToggleFullscreen();       				// Toggle Fullscreen/Windowed (Works On Linux/BeOS Only)
Begin
   if( (not assigned(Screen.Surface)) or (SDL_WM_ToggleFullScreen(Screen.Surface)<>1)) then begin
      Log.LogError( Format( 'Unable to toggle fullscreen: %s', [SDL_GetError] ),   'ToggleFullScreen' );
   end;
end;

function TSDL_System.WinMain(hInstance: HINST; hPrevInstance: HINST; pCmdLine: PChar; nCmdShow: integer): integer; stdcall;
begin
   result:=0;

   if doGL then LoadOpenGL;

   if(SDL_Init(SDL_INIT_VIDEO)<0) then begin
      Log.LogError( Format( 'Unable to open SDL: %s', [SDL_GetError] ),   'WinMain' );
      result:=1;
      TerminateApplication;
   end;

   Screen.videoInfo := SDL_GetVideoInfo;

   if ( not assigned(Screen.videoInfo) ) then begin
      Log.LogError( Format( 'Video query failed : %s', [SDL_GetError] ),  'Main' );
      TerminateApplication;
   end;

   Screen.VideoFlags := SDL_DOUBLEBUF or SDL_HWPALETTE or SDL_RESIZABLE;	// We Want A Hardware Surface, Double Buffering Feature And Special OpenGLBlit Mode

   if doGL then begin
      Screen.Videoflags:=Screen.Videoflags or SDL_OPENGL;

      SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE,16);
      if screen.bStencil then SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE,1);
   end;

  // This checks to see if surfaces can be stored in memory
   if ( Screen.VideoInfo.hw_available <> 0 ) then  Screen.VideoFlags := Screen.VideoFlags or SDL_HWSURFACE
      else Screen.VideoFlags := Screen.VideoFlags or SDL_SWSURFACE;

  // This checks if hardware blits can be done * /
   if ( Screen.VideoInfo.blit_hw <> 0 ) then Screen.Videoflags := Screen.VideoFlags or SDL_HWACCEL;

   if Screen.Fullscreen_ASK then begin
      if(MessageDlg('Use fullscreen mode?', mtConfirmation, [MBYES,MBNO],0) = id_Yes) then begin
         Screen.Videoflags:=Screen.VideoFlags or SDL_FULLSCREEN;
         Screen.Fullscreen:=true;
      end else Screen.Fullscreen:=false;
   end;

   if Screen.Fullscreen then begin						// Now, We Can Decide To Always Launch Out Application Fullscreen
      Screen.VideoFlags:=Screen.VideoFlags or SDL_FULLSCREEN;			// If So, We Always Need The Fullscreen Video Init Flag
      SDL_ShowCursor(0);
   end;

   if doGL then begin
      if not CreateWindowGL then begin		                                // Our Video Flags Are Set, We're Creating The Window
         Log.LogError( Format( 'Unable to open screen surface: %s', [SDL_GetError] ),   'WinMain' );
         result:=1;
         TerminateApplication;
      end;
   end;

   Screen.videoInfo := SDL_GetVideoInfo;


   SDL_WM_SetCaption(pchar(App_Name), nil);					// We're Setting The Window Caption

   if (assigned(PARInitialize)) and (not PARInitialize) then begin		// Now We're Initting The Application
      Log.LogError( Format( 'App init failed: %s', [SDL_GetError] ),   'WinMain' );
      result:=1;
      TerminateApplication;
   end;
        
   if (assigned(PARInitGL) and doGL) and (not ParInitGL) then begin		// We're Calling The OpenGL Init Function
      Log.LogError( Format( 'Cant init GL: %s', [SDL_GetError] ),   'WinMain' );
      result:=1;
      TerminateApplication;
   end;

   if doFont and (not SetupFont) then begin
      Log.LogError( Format( 'Font init failed: %s', [SDL_GetError] ),   'WinMain' );
      result:=1;
      TerminateApplication;
   end;

   if doGlFont and (not SetupGlFont) then begin
      Log.LogError( Format( 'GL Font init failed: %s', [SDL_GetError] ),   'WinMain' );
      result:=1;
      TerminateApplication;
   end;

   if not InitTimers then begin	                				// We Call The Timers Init Function
      Log.LogError( Format( 'Cant init the timers: %s', [SDL_GetError] ),   'WinMain' );
      result:=1;
      TerminateApplication;
   end;

   SDL_EventState(SDL_MOUSEMOTION,SDL_ENABLE);                                  // Capture mouse events
   SDL_EventState(SDL_MOUSEBUTTONDOWN,SDL_ENABLE);
   SDL_EventState(SDL_MOUSEBUTTONUP,SDL_ENABLE);


   isProgramLooping := true;
   while(isProgramLooping) do begin						// And While It's looping

      if SDL_PollEvent(@E)=1 then Handle_SDL_Event;

      if( not AppStatus.Visible) then begin					// If The Application Is Not Visible
         SDL_WaitEvent(nil);							// Leave The CPU Alone, Don't Waste Time, Simply Wait For An Event
      end else begin 								// Otherwhise
         if assigned(PARUpdate) then ParUpdate; 	                        // And Update The Motions And Data
         if assigned(PARDraw) then ParDraw();  					// Do The Drawings!
         if doGL then SDL_GL_SwapBuffers() 					// And Swap The Buffers (We're Double-Buffering, Remember?)
            else SDL_Flip(Screen.Surface);
      end;

     if doFPS then Inc(FPS_Count);                                                        // Increment FPS Counter

      if TimerGetTime > NextTimer then begin
         ElapsedTimer:=LastTimer-TimerGetTime;
         LastTimer:=NextTimer;
         NextTimer:=TimerGetTime+1000;

         if doFPS then begin
            FPS_Last_Count:=FPS_Count;                                                        // Increment FPS Counter
            FPS_String:='FPS [' + intToStr(FPS_Count) + ']';
            SDL_WM_SetCaption(pchar(App_Name + '   ' +FPS_String), nil);	// We're Setting The Window Caption
            FPS_Count := 0;
         end;

      end;


   end;
   TerminateApplication;
end;

initialization
   PAR_System:=TSDL_System.create;

finalization
   PAR_System.free;
end.
