program zeldaSDL;
// PAR SDL Zelda was created using the PAR SDL Framework
// Created by Paul Robello by examples and demos from but not limited to:
// nehe.gamedev.net, www.delphi-jedi.org, romka.demonews.com, www.nutty.org.
// Most code has been heavily modified for speed and integration.
// PAR SDL Framework is an always changing project that may not be backward
// compatable with other projects.
// Questions, comments, ravs, flames email paulr@cal.net

uses
  dialogs,
  SysUtils,
  Logger,
  sdl,
  smpeg,
  OpenGL12,
  ParSdlMain in '..\parsdlmain.pas',
  parsdlgl in '..\parsdlgl.pas',
  ZeldaTiles in 'ZeldaTiles.pas',
  flaredef in 'flaredef.pas',
  compases in 'compases.pas',
  monsterdef in 'monsterdef.pas',
  linkdef in 'linkdef.pas';


procedure mydraw; forward;
procedure mapdraw; forward;

var debug : boolean = false;                                                    // show tile debug info (slow)
    doFrameLimit : boolean = true;                                              // slow down fast parts
    mainTick : single = 0.0;                                                    // max time to render main tile loop
    mpeg:PSMpeg;                                                                // background music
    mpegInfo:SMPEG_Info;
    BG_Sound : boolean = true;                                                  // background music on or off


procedure setUpLight();
begin
   light0.Pos[0] := 10;                                   			// Light Position
   light0.Pos[1] := 2.5;
   light0.Pos[2] := 0;
   light0.Pos[3] := 1;

   light0.Diffuse[0] := 0.80;           	        	                // Diffuse Light Values
   light0.Diffuse[1] := 0.80;
   light0.Diffuse[2] := 0.80;
   light0.Diffuse[3] := 0.80;

   light0.Specular[0]:=0;                                                       // no specular
   light0.Specular[1]:=0;
   light0.Specular[2]:=0;
   light0.Specular[3]:=0;

   light0.Shininess:=65;

   light0.Ambient[0] := 0.1;                                                    // Ambient Light Values
   light0.Ambient[1] := 0.1;
   light0.Ambient[2] := 0.1;
   light0.Ambient[3] := 1.0;

   SetLight(@Light0);                                                           // let their be light
end;

procedure SetUpCamera();
begin
   Camera.Pos.x:=0;                                                             // move world around camera
   Camera.Pos.y:=0;
   Camera.Pos.z:=0;

   Camera.Rot.x:=0;
   Camera.Rot.y:=0;
   Camera.Rot.z:=0;

   camera.Look.x:=CameraLookCOS[link.dir];
   Camera.Look.y:=0;
   camera.Look.z:=CameraLookSIN[link.dir];
end;

procedure mySetupFog;
begin
   fog.color[0]:=0.71;
   fog.color[1]:=0.71;
   fog.color[2]:=0.75;
   fog.Filter:=2;

   if isGeforce then begin
      fog.Front:=VIEWDIST/2;
      fog.Back:=VIEWDIST-2;
   end else begin
      fog.Front:=1;
      fog.Back:=VIEWDIST-2;
   end;
   if fog.Enabled then PARsetupfog(@fog);
end;

function MyInitGL : boolean;		  			                // Any OpenGL Initialization Code Goes Here
begin
   result:=true;

   defaultInitGl;                                                               // call default gl setup stuff

   SetUpCamera;

   SetupLight;

   fog.Enabled:=true;
   mySetupFog;

   LoadTextures;

   loadmap('map.txt');

   buildtiles;

   buildMonsters;
end;

procedure attack;
begin
   //
end;

procedure MyKeyUp(key : TSDLKey);
begin
   case key of
      SDLK_Z : begin
         if ViewDist>4 then dec(ViewDist);
         BuildTileLookup;
         mySetupFog;
      end;
      SDLK_X : begin
         if ViewDist<ViewDistMax then inc(ViewDist);
         BuildTileLookup;
         mySetupFog;
      end;
      SDLK_B : begin
         BG_Sound:=not BG_Sound;
         if not BG_Sound then SMPEG_stop(mpeg) else SMPEG_play(mpeg);
      end;
      SDLK_D : begin
         debug:=not debug;
      end;
      SDLK_W : begin
         if PAR_System.Screen.PollyFill=GL_FILL then PAR_System.Screen.PollyFill:=GL_LINE else PAR_System.Screen.PollyFill:=GL_FILL;
      end;
      SDLK_C : begin
         ShowCompass:=not ShowCompass;
      end;
      SDLK_T : begin
         inc(CurrCompass);
         if CurrCompass>=MaxCompass then CurrCompass:=0;
      end;
      SDLK_F : begin
         fog.Enabled:=not fog.Enabled;
         PARSetupFog(@fog);
      end;
      SDLK_L : begin
         doFlare:=not doFlare;
      end;
      SDLK_R : begin
         doFrameLimit:=not doFrameLimit;
      end;
      SDLK_P : begin
         PAR_System.doPause:=not PAR_System.doPause;
      end;
      SDLK_TAB : begin
         ShowMap:=not ShowMap;
         map.xoff:=0;
         map.yoff:=0;
         map.zoom:=0;
         map.transition:=0;
         map.shroud:=6;
         if not ShowMap then  begin
            PAR_System.ParDraw:=MyDraw;
            PAR_System.doPause:=false;
         end else begin
            PAR_System.doPause:=true;
         end;
      end;

   end;
   if not PAR_System.doPause then begin
      if link.state>=lstNormal then begin
         case key of
            SDLK_LCTRL : begin
               attack;
            end;
         end;
      end;
   end;
end;


procedure CheckCollision;
var HitMonster : boolean;
    m : longint;
begin
   with link do begin
      if (lx<0) or (lx>map.xmax) then lx:=oldLx;
      if (ly<0) or (ly>map.ymax) then ly:=oldLy;
      ilx:=trunc(lx);
      ily:=trunc(ly);
      hitMonster:=false;
      for m:=0 to numMonsters-1 do begin
         if (abs(nlx-monster[m].pos[0])<0.5) and (abs(nly-monster[m].pos[1])<0.5) then hitMonster:=true;
      end;

      if not (map.data[nlx,nly] in [5,6,7,36,41,48,67,68,87]) or hitMonster then begin
         lx:=oldLx;
         ly:=oldLy;
         ilx:=trunc(lx);
         ily:=trunc(ly);
      end;
   end;
end;

procedure DoInput;
begin
   with link do begin

      oldLx:=lx;
      oldLy:=ly;
      olddir:=dir;

      if (PAR_System.Keys[SDLK_M]<>0) then begin
         mainTick:=0.0;
      end;

      if link.state>=lstNormal then begin
         if showMap then begin
            if (PAR_System.Keys[SDLK_UP]<>0) then begin
               inc(map.yoff);
            end;
            if (PAR_System.Keys[SDLK_DOWN]<>0) then begin
               dec(map.yoff);
            end;
            if (PAR_System.Keys[SDLK_LEFT]<>0) then begin
               dec(map.xoff);
            end;
            if (PAR_System.Keys[SDLK_RIGHT]<>0) then begin
               inc(map.xoff);
            end;
            if (PAR_System.Keys[SDLK_PAGEUP]<>0) then begin
               inc(map.zoom);
            end;
            if (PAR_System.Keys[SDLK_PAGEDOWN]<>0) then begin
               dec(map.zoom);
            end;
         end else begin
            if not PAR_System.doPause then begin
               if (PAR_System.Keys[SDLK_UP]<>0) then begin
                  lx:=lx-(PAR_System.COS_LOOK[dir]*0.1);
                  ly:=ly+(PAR_System.SIN_LOOK[dir]*0.1);
                  inc(viewbob,10);
                  if viewbob>360 then viewbob:=viewbob-360;

                  nlx:=round(lx-PAR_SYSTEM.COS_LOOK[dir]);
                  nly:=round(ly+PAR_SYSTEM.SIN_LOOK[dir]);

                  CheckCollision;
               end;

               if (PAR_System.Keys[SDLK_DOWN]<>0) then begin
                  lx:=lx+(PAR_System.COS_LOOK[dir]*0.1);
                  ly:=ly-(PAR_System.SIN_LOOK[dir]*0.1);
                  inc(viewbob,10);
                  if viewbob>360 then viewbob:=viewbob-360;

                  nlx:=round(lx+PAR_SYSTEM.COS_LOOK[dir]);
                  nly:=round(ly-PAR_SYSTEM.SIN_LOOK[dir]);

                  CheckCollision;
               end;

               if (PAR_System.Keys[SDLK_LEFT]<>0) then begin
                  dir:=dir-2;
                  if dir<0 then dir:=dir+360;
                  camera.Look.x:=CameraLookCOS[link.dir];
                  camera.Look.z:=CameraLookSIN[link.dir];
               end;

               if (PAR_System.Keys[SDLK_RIGHT]<>0) then begin
                  dir:=dir+2;
                  if dir > 360 then dir:=dir-360;
                  camera.Look.x:=CameraLookCOS[link.dir];
                  camera.Look.z:=CameraLookSIN[link.dir];
               end;
            end;
         end;
      end;
   end;
end;

procedure MyUpdate;
begin
   if(assigned(PAR_System.Keys)) then begin
      if(PAR_System.Keys[SDLK_ESCAPE]<>0) then begin
         PAR_System.TerminateApplication;
      end;
      if(PAR_System.Keys[SDLK_F1]<>0) then begin
         PAR_System.ToggleFullscreen();	  					// Use SDL Function To Toggle Fullscreen Mode (But Not In Windows :) )
      end;
      doInput;
   end;
end;

procedure MapDraw();
var x,y,tx,ty : longint;
    lt : byte;
begin
   glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
   glDisable(GL_DEPTH_TEST);
   glEnable(GL_TEXTURE_2D);
   glEnable(GL_COLOR_MATERIAL);
   glDisable(GL_FOG);
   glDisable(GL_BLEND);
   glDisable(GL_DEPTH_TEST);
   glDisable(GL_CULL_FACE);
   glLoadIdentity;
   gluLookAt(0,0,-20-(map.zoom*2), 0,0,0, 0,1,0);

   glrotatef(180-map.transition,1,0,0);
   if map.transition<180-1 then inc(map.transition,3);
   gltranslatef(0,-1+(map.transition-90)*0.01,0);
   gltranslatef(frac(link.lx*2),-frac(link.ly*2),0);

   for y:=5+map.zoom downto -5-map.zoom do begin
      for x:=-7-map.zoom to 7+map.zoom do begin
         tx:=link.ilx+x+map.xoff;
         ty:=link.ily+y+map.yoff;

         if (tx<0) or (ty<0) or (tx>map.xmax) or (ty>map.ymax) then continue;
         if sqrt(sqr(tx-link.ilx)+sqr(ty-link.ily))>3 then glcolor4f(0.5,0.5,0.5,1) else glcolor4f(1,1,1,1);
         glPushMatrix();			 				// Store The Modelview Matrix
            glTranslatef(-x,y,0);        				        // Position The Tile
            glCallList(tiles_2d+map.data[tx,ty]);				// Draws The Display List Text

            if sqrt(sqr(tx-link.ilx)+sqr(ty-link.ily))>map.shroud then glcolor4f(0.5,0.5,0.5,0.35) else glcolor4f(1,1,1,0.35); // draw fog
            glDisable(GL_TEXTURE_2D);
            glEnable(GL_BLEND);
            glBegin(GL_QUADS);
               glVertex3f(-0.5,-0.5,0.001);                                     // Texture / Vertex Coord (Bottom Left)}
               glVertex3f( 0.5,-0.5,0.001);                                     // Texutre / Vertex Coord (Bottom Right)
               glVertex3f( 0.5, 0.5,0.001);                                     // Texture / Vertex Coord (Top Right)
               glVertex3f(-0.5, 0.5,0.001);                                     // Texture / Vertex Coord (Top Left)
            glEnd();
            glDisable(GL_BLEND);
            glEnable(GL_TEXTURE_2D);
         glPopMatrix();								// Restore The Old Projection Matrix
      end;
   end;

   lt:=107;                                                                     // Rotate 2d link to match 3d link
   if (link.dir>=45)  and (link.dir<=135) then lt:=105;
   if (link.dir>=136) and (link.dir<=225) then lt:=109;
   if (link.dir>=225) and (link.dir<=315) then lt:=103;

   gltranslatef(-frac(link.lx*2)+map.xoff,(frac(link.ly*2)-map.yoff),0.01);     // draw 2d link
   glEnable(GL_BLEND);
   glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
   glCallList(tiles_2d+lt);
   for x:=0 to numMonsters-1 do begin
      if Monster[x].state<mstStop then continue;                                // only draw live monsters
      if sqrt(sqr(Monster[x].pos[0]-link.ilx)+sqr(Monster[x].pos[1]-link.ily))>map.shroud then continue; // only draw monsters in proximity
      glPushMatrix();
      gltranslatef(Monster[x].pos[0]-link.ilx,Monster[x].pos[1]-link.ily,0);
      glCallList(tiles_2d+111);				                        // Draws The Display List Text
      glPopMatrix();
   end;

   if debug then begin                                                          // show link/tile info and coords (slow)
      glprint(format('lx(%.2f,%.2f) nx(%d,%d) dir[%d] TT=%.2f, FPS=%d, gf=%d',[link.lx,link.ly,link.nlx,link.nly,link.dir,mainTick,trunc(PAR_SYSTEM.FPS_Last_Count),byte(isGeforce)]));  // standard debug info
   end;
end;

procedure MyDraw();								// Our Drawing Code
var r,tx,ty,tc : longint;
    tick : single;
begin


   tick:=PAR_System.timerGetTime;

   glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

   DrawSky;

   glLoadIdentity;                                                              // Reset The Modelview Matrix
   setcamera(@camera);
   SetLight(@Light0);

   if showMap then begin
      if map.transition<90 then inc(map.transition,3) else PAR_System.ParDraw:=MapDraw;
      glscalef(1,1-(map.transition*0.01),1);
   end;

   if (link.dir>155) and (link.dir<210) then DrawSun;

   glEnable(GL_LIGHTING);
   glEnable(GL_DEPTH_TEST);

   for r:=0 to numMonsters-1 do  monster[r].draw(@link);

   gltranslatef(-frac(link.lx), ViewBobLookSIN[link.viewbob],frac(link.ly));
   glrotatef(-90,1,0,0);


   tc:=0;                                                                       // counter to see how many tiles are being drawn
   for r:=1 to TileLook[link.dir][0].x+1 do begin
      if not PAR_System.isProgramLooping then exit;
      tx:=link.ilx+TileLook[link.dir][r].x;                                     // map tile pos x
      ty:=link.ily+TileLook[link.dir][r].y;                                     // map tile pos y
      if (tx<0) or (ty<0) or (tx>map.xmax) or (ty>map.ymax) then continue;      // dont draw out of map bounds

      inc(tc);
      if PAR_System.screen.PollyFill=GL_LINE then glPolygonMode( GL_FRONT_AND_BACK, PAR_System.screen.PollyFill );       // solid or wire frame
      if fog.Enabled then glEnable(GL_FOG);
      glPushMatrix();								// Store The Modelview Matrix
         glTranslatef(TileLook[link.dir][r].x,TileLook[link.dir][r].y,0);	// Position The Tile
         glCallList(tiles_3D+map.data[tx,ty]);      				// Draws The Display List Text
      glPopMatrix();								// Restore The Old Projection Matrix


      if PAR_System.screen.PollyFill=GL_LINE then glPolygonMode( GL_FRONT_AND_BACK, GL_FILL ); // solid

      if debug then begin                                                       // show tile info and coords (slow)
         if fog.Enabled then glDisable(GL_FOG);
         glprintTGA(TileLook[link.dir][r].x,TileLook[link.dir][r].y,-0.0625*7,0,format('tp(%d,%d,%d)',[tx,ty,map.data[tx,ty]]));
         if fog.Enabled then glEnable(GL_FOG);
      end;
   end;

   if ShowCompass then begin                                                        // draw compass last always visible unless toggled off
      case currCompass of
         0 : doCompass0(link.dir);
         1 : doCompass1(link.dir);
         2 : doCompass2(link.dir);
      end;
   end;

   Materials.SetMaterial('waterc');
   gltranslatef(3,0,-0.5);

   DrawHud;                                                                     // link stats and wepon info

   if doFlare and (link.dir>155) and (link.dir<210) then calcflare;             // draw lense flare


   if debug then begin                                                          // show link/tile info and coords (slow)
//      glprint(0,64,0,format('lx(%.2f,%.2f) nx(%d,%d) dir[%d] Tc=%d, TT=%.2f, FPS=%d, gf=%d',[link.lx,link.ly,tx,ty,link.dir,tc,mainTick,trunc(PAR_SYSTEM.FPS_Last_Count),byte(isGeforce)]));  // standard debug info
      glprint(format('lx(%.2f,%.2f) nx(%d,%d) dir[%d] Tc=%d, TT=%.2f, FPS=%d, gf=%d',[link.lx,link.ly,link.nlx,link.nly,link.dir,tc,mainTick,trunc(PAR_SYSTEM.FPS_Last_Count),byte(isGeforce)]));  // standard debug info
   end;

   glFlush();


   tick:=PAR_System.timerGetTime-tick;
   if (tick>mainTick) then mainTick:=tick;                                      // max time taken for single tile draw proccess

   if doFrameLimit and (not ShowMap) then begin
      r:=trunc((30-ViewDist)*1.25-tick);
      if r>0 then sleep(r);
   end;
end;

function MyInitialize() : boolean;						// Any Application & User Initialization Code Goes Here
begin
   result:=true;
   ViewDist:=10;
   if isGeforce then ViewDist:=ViewDist+3;
   BuildTileLookup;
   link:=tlink.create;

      if (SDL_Init(SDL_INIT_AUDIO) < 0) then begin
         Log.LogError(Format('Couldn''t initialize SDL : %s',[SDL_GetError]), '[MyInitalize]');
      end else begin
         mpeg := SMPEG_new(pchar('themesong1.mp3'), @mpegInfo, 1);
         if not assigned(mpeg) then begin
            Log.LogError(Format('Couldn''t initialize SDL : %s',[SDL_GetError]), '[MyInitalize]');
         end else begin
            SMPEG_enableaudio(mpeg, 1);
            SMPEG_enablevideo(mpeg, 0);
            SMPEG_setvolume(mpeg, 100);
            SMPEG_Loop(mpeg,1);
            if BG_Sound then SMPEG_play(mpeg);
         end;
      end;
end;

procedure MyDeinitialize();
begin;
   if glIsList(tiles_2D) then glDeleteLists(tiles_2D,112); 				// Delete All 111 Tile lists
   if glIsList(tiles_3D) then glDeleteLists(tiles_3D,112); 				// Delete All 111 Tile lists
   if glIsList(sunlist) then glDeleteLists(sunlist,1);				// Delete sun list

   disposeMonsters;
   link.Destroy;

   if assigned(mpeg) then SMPEG_delete(mpeg);                                   // free background music

end;

begin
   randomize;

   PAR_System.Screen.Width:=1024;
   PAR_System.Screen.Height:=768;
   PAR_System.Screen.BPP:=32;

   PAR_System.ParInitialize:=MyInitialize;
   PAR_System.ParInitGl:=MyInitGl;                                              // Assign custom functions
   PAR_System.ParUpdate:=MyUpdate;
   PAR_System.ParDraw:=MyDraw;
   PAR_System.ParDeinitialize:=MyDeinitialize;
   PAR_System.PARKeyUp:=MyKeyUp;

{$WARNINGS OFF}
   PAR_System.WinMain( hInstance, hPrevInst, CmdLine, CmdShow );                // Start program
{$WARNINGS ON}
end.

